@@ -1,9 +1,10 @@
 use std::env;
 use std::fs;
 use std::io::{self, Write};
-use clap::Shell;
+use std::path::Path;
+use clap_complete::Shell;
 
-mod sq_cli {
+pub mod sq_cli {
     include!("src/sq_cli.rs");
 }
 
@@ -13,11 +14,10 @@ fn main() {
     // XXX: Revisit once
     // https://github.com/rust-lang/rust/issues/44732 is stabilized.
 
-    let mut sq = sq_cli::configure(
-        clap::App::new("sq").set_term_width(80),
-        cfg!(feature = "net"),
-        cfg!(feature = "autocrypt"),
-    );
+    subplot_build::codegen(Path::new("sq-subplot.md"))
+        .expect("failed to generate code with Subplot");
+
+    let mut sq = sq_cli::build().term_width(80);
     let mut main = fs::File::create("src/sq-usage.rs").unwrap();
     dump_help(&mut main,
               &mut sq,
@@ -35,14 +35,16 @@ fn main() {
     };
     fs::create_dir_all(&outdir).unwrap();
     let mut sq = sq_cli::build();
+
     for shell in &[Shell::Bash, Shell::Fish, Shell::Zsh, Shell::PowerShell,
                    Shell::Elvish] {
-        sq.gen_completions("sq", *shell, &outdir);
-    }
+        let path = clap_complete::generate_to(*shell, &mut sq, "sq", &outdir).unwrap();
+        println!("cargo:warning=completion file is generated: {:?}", path);
+    };
 }
 
 fn dump_help(sink: &mut dyn io::Write,
-             sq: &mut clap::App,
+             sq: &mut clap::Command,
              cmd: Vec<String>,
              heading: &str)
              -> io::Result<()>
@@ -64,12 +66,12 @@ fn dump_help(sink: &mut dyn io::Write,
         .chain(std::iter::once("--help"))
         .collect::<Vec<_>>();
 
-    let help = sq.get_matches_from_safe_borrow(&args)
+    let help = sq.try_get_matches_from_mut(&args)
         .unwrap_err().to_string();
 
     writeln!(sink, "//! ```text")?;
-    for line in help.trim_end().split("\n").skip(1) {
-        if line == "" {
+    for line in help.trim_end().split('\n').skip(1) {
+        if line.is_empty() {
             writeln!(sink, "//!")?;
         } else {
             writeln!(sink, "//! {}", line.trim_end())?;
@@ -79,7 +81,7 @@ fn dump_help(sink: &mut dyn io::Write,
 
     // Recurse.
     let mut found_subcommands = false;
-    for subcmd in help.split("\n").filter_map(move |line| {
+    for subcmd in help.split('\n').filter_map(move |line| {
         if line == "SUBCOMMANDS:" {
             found_subcommands = true;
             None
@@ -87,7 +89,7 @@ fn dump_help(sink: &mut dyn io::Write,
             if line.chars().nth(4).map(|c| ! c.is_ascii_whitespace())
                 .unwrap_or(false)
             {
-                line.trim_start().split(" ").nth(0)
+                line.trim_start().split(' ').next()
             } else {
                 None
             }
