@@ -1,5 +1,7 @@
 use std::time::{SystemTime, Duration};
 
+use anyhow::Context;
+
 use sequoia_openpgp as openpgp;
 use openpgp::Result;
 use openpgp::cert::prelude::*;
@@ -11,36 +13,49 @@ use openpgp::types::SignatureType;
 
 use crate::Config;
 use crate::parse_duration;
+use crate::parse_notations;
 use crate::SECONDS_IN_YEAR;
+use crate::commands::get_certification_keys;
+use crate::commands::GetKeysOptions;
 
-pub fn certify(config: Config, m: &clap::ArgMatches)
+use crate::sq_cli::CertifyCommand;
+
+pub fn certify(config: Config, c: CertifyCommand)
     -> Result<()>
 {
-    let certifier = m.value_of("certifier").unwrap();
-    let cert = m.value_of("certificate").unwrap();
-    let userid = m.value_of("userid").unwrap();
+    let certifier = c.certifier;
+    let cert = c.certificate;
+    let userid = c.userid;
 
     let certifier = Cert::from_file(certifier)?;
+    let private_key_store = c.private_key_store;
     let cert = Cert::from_file(cert)?;
-    let vc = cert.with_policy(&config.policy, None)?;
 
-    let trust_depth: u8 = m.value_of("depth")
-        .map(|s| s.parse()).unwrap_or(Ok(0))?;
-    let trust_amount: u8 = m.value_of("amount")
-        .map(|s| s.parse()).unwrap_or(Ok(120))?;
-    let regex = m.values_of("regex").map(|v| v.collect::<Vec<_>>())
-        .unwrap_or(vec![]);
-    if trust_depth == 0 && regex.len() > 0 {
+    let trust_depth: u8 = c.depth;
+    let trust_amount: u8 = c.amount;
+    let regex = c.regex;
+    if trust_depth == 0 && !regex.is_empty() {
         return Err(
             anyhow::format_err!("A regex only makes sense \
                                  if the trust depth is greater than 0"));
     }
 
-    let local = m.is_present("local");
-    let non_revocable = m.is_present("non-revocable");
-    let expires = m.value_of("expires");
-    let expires_in = m.value_of("expires-in");
+    let local = c.local;
+    let non_revocable = c.non_revocable;
 
+    let time = if let Some(t) = c.time {
+        let time = SystemTime::from(
+            crate::parse_iso8601(&t, chrono::NaiveTime::from_hms(0, 0, 0))
+                .context(format!("Parsing --time {}", t))?);
+        Some(time)
+    } else {
+        None
+    };
+
+    let expires = c.expires;
+    let expires_in = c.expires_in;
+
+    let vc = cert.with_policy(&config.policy, time)?;
 
     // Find the matching User ID.
     let mut u = None;
@@ -90,6 +105,11 @@ pub fn certify(config: Config, m: &clap::ArgMatches)
         builder = builder.set_revocable(false)?;
     }
 
+    // Creation time.
+    if let Some(time) = time {
+        builder = builder.set_signature_creation_time(time)?;
+    }
+
     match (expires, expires_in) {
         (None, None) =>
             // Default expiration.
@@ -102,7 +122,7 @@ pub fn certify(config: Config, m: &clap::ArgMatches)
             let now = builder.signature_creation_time()
                 .unwrap_or_else(std::time::SystemTime::now);
             let expiration = SystemTime::from(
-                crate::parse_iso8601(t, chrono::NaiveTime::from_hms(0, 0, 0))?);
+                crate::parse_iso8601(&t, chrono::NaiveTime::from_hms(0, 0, 0))?);
             let validity = expiration.duration_since(now)?;
             builder = builder.set_signature_creation_time(now)?
                 .set_signature_validity_period(validity)?;
@@ -111,38 +131,37 @@ pub fn certify(config: Config, m: &clap::ArgMatches)
             // The default is no expiration; there is nothing to do.
             (),
         (None, Some(d)) => {
-            let d = parse_duration(d)?;
+            let d = parse_duration(&d)?;
             builder = builder.set_signature_validity_period(d)?;
         },
         (Some(_), Some(_)) => unreachable!("conflicting args"),
     }
 
-    // Each --notation takes two values.  The iterator returns them
-    // one at a time, however.
-    if let Some(mut n) = m.values_of("notation") {
-        while let Some(name) = n.next() {
-            let value = n.next().unwrap();
+    let notations = parse_notations(c.notation.unwrap_or_default())?;
+    for (critical, n) in notations {
+        builder = builder.add_notation(
+            n.name(),
+            n.value(),
+            NotationDataFlags::empty().set_human_readable(),
+            critical)?;
+    };
 
-            let (critical, name) = if name.len() > 0
-                && Some('!') == name.chars().next()
-            {
-                (true, &name[1..])
-            } else {
-                (false, name)
-            };
-
-            builder = builder.add_notation(
-                name,
-                value,
-                NotationDataFlags::empty().set_human_readable(),
-                critical)?;
-        }
+    let mut options = Vec::new();
+    if c.allow_not_alive_certifier {
+        options.push(GetKeysOptions::AllowNotAlive);
+    }
+    if c.allow_revoked_certifier {
+        options.push(GetKeysOptions::AllowRevoked);
     }
 
-
     // Sign it.
-    let mut signer = certifier.primary_key().key().clone()
-        .parts_into_secret()?.into_keypair()?;
+    let signers = get_certification_keys(
+        &[certifier], &config.policy,
+        private_key_store.as_deref(),
+        time,
+        Some(&options))?;
+    assert_eq!(signers.len(), 1);
+    let mut signer = signers.into_iter().next().unwrap();
 
     let certification = builder
         .sign_userid_binding(
@@ -160,8 +179,10 @@ pub fn certify(config: Config, m: &clap::ArgMatches)
 
     // And export it.
     let mut message = config.create_or_stdout_pgp(
-        m.value_of("output"),
-        m.is_present("binary"), sequoia_openpgp::armor::Kind::PublicKey)?;
+        c.output.as_deref(),
+        c.binary,
+        sequoia_openpgp::armor::Kind::PublicKey,
+    )?;
     cert.serialize(&mut message)?;
     message.finalize()?;
 
