@@ -9,9 +9,11 @@ use self::openpgp::packet::prelude::*;
 use self::openpgp::packet::header::CTB;
 use self::openpgp::packet::{Header, header::BodyLength, Signature};
 use self::openpgp::packet::signature::subpacket::{Subpacket, SubpacketValue};
-use self::openpgp::crypto::{SessionKey, S2K};
+use self::openpgp::crypto::S2K;
 use self::openpgp::parse::{map::Map, Parse, PacketParserResult};
 
+use crate::sq_cli::CliSessionKey;
+
 #[derive(Debug)]
 pub enum Kind {
     Message {
@@ -52,9 +54,11 @@ impl Convert<chrono::DateTime<chrono::offset::Utc>> for Timestamp {
     }
 }
 
+#[allow(clippy::redundant_pattern_matching)]
 pub fn dump<W>(input: &mut (dyn io::Read + Sync + Send),
                output: &mut dyn io::Write,
-               mpis: bool, hex: bool, sk: Option<&SessionKey>,
+               mpis: bool, hex: bool,
+               sk: Option<&CliSessionKey>,
                width: W)
                -> Result<Kind>
     where W: Into<Option<usize>>
@@ -83,27 +87,30 @@ pub fn dump<W>(input: &mut (dyn io::Read + Sync + Send),
             }
             Packet::SEIP(_) if sk.is_some() => {
                 message_encrypted = true;
-                let sk = sk.as_ref().unwrap();
-                let mut decrypted_with = None;
-                for algo in 1..20 {
-                    let algo = SymmetricAlgorithm::from(algo);
-                    if let Ok(size) = algo.key_size() {
-                        if size != sk.len() { continue; }
-                    } else {
-                        continue;
-                    }
+                let sk = sk.unwrap();
+                let decrypted_with = if let Some(algo) = sk.symmetric_algo {
+                    // We know which algorithm to use, so only try decrypting
+                    // with that one.
+                    pp.decrypt(algo, &sk.session_key).is_ok().then(|| algo)
+                } else {
+                    // We don't know which algorithm to use,
+                    // try to find one that decrypts the message.
+                    (1u8..=19)
+                        .map(SymmetricAlgorithm::from)
+                        .find(|algo| pp.decrypt(*algo, &sk.session_key).is_ok())
+                };
 
-                    if let Ok(_) = pp.decrypt(algo, sk) {
-                        decrypted_with = Some(algo);
-                        break;
-                    }
-                }
                 let mut fields = Vec::new();
-                fields.push(format!("Session key: {}", hex::encode(sk)));
+                fields.push(format!("Session key: {}", &sk.display_sensitive()));
                 if let Some(algo) = decrypted_with {
                     fields.push(format!("Symmetric algo: {}", algo));
                     fields.push("Decryption successful".into());
                 } else {
+                    if let Some(algo) = sk.symmetric_algo {
+                        fields.push(format!(
+                            "Indicated Symmetric algo: {}", algo
+                        ));
+                    };
                     fields.push("Decryption failed".into());
                 }
                 Some(fields)
@@ -121,14 +128,14 @@ pub fn dump<W>(input: &mut (dyn io::Read + Sync + Send),
                     unreachable!()
                 };
 
-                let _ = pp.decrypt(algo, sk);
+                let _ = pp.decrypt(algo, &sk.session_key);
 
                 let mut fields = Vec::new();
-                fields.push(format!("Session key: {}", hex::encode(sk)));
-                if pp.encrypted() {
-                    fields.push("Decryption failed".into());
-                } else {
+                fields.push(format!("Session key: {}", sk.display_sensitive()));
+                if pp.processed() {
                     fields.push("Decryption successful".into());
+                } else {
+                    fields.push("Decryption failed".into());
                 }
                 Some(fields)
             },
@@ -185,10 +192,10 @@ impl Node {
     fn new(header: Header, packet: Packet, map: Option<Map>,
            additional_fields: Option<Vec<String>>) -> Self {
         Node {
-            header: header,
-            packet: packet,
-            map: map,
-            additional_fields: additional_fields,
+            header,
+            packet,
+            map,
+            additional_fields,
             children: Vec::new(),
         }
     }
@@ -211,8 +218,8 @@ pub struct PacketDumper {
 impl PacketDumper {
     pub fn new(width: usize, mpis: bool) -> Self {
         PacketDumper {
-            width: width,
-            mpis: mpis,
+            width,
+            mpis,
             root: None,
         }
     }
@@ -225,21 +232,19 @@ impl PacketDumper {
         if self.root.is_none() {
             assert_eq!(depth, 0);
             self.root = Some(node);
+        } else if depth == 0 {
+            let root = self.root.take().unwrap();
+            self.dump_tree(output, "", &root)?;
+            self.root = Some(node);
         } else {
-            if depth == 0 {
-                let root = self.root.take().unwrap();
-                self.dump_tree(output, "", &root)?;
-                self.root = Some(node);
-            } else {
-                self.root.as_mut().unwrap().append(depth - 1, node);
-            }
+            self.root.as_mut().unwrap().append(depth - 1, node);
         }
         Ok(())
     }
 
     pub fn flush(&self, output: &mut dyn io::Write) -> Result<()> {
         if let Some(root) = self.root.as_ref() {
-            self.dump_tree(output, "", &root)?;
+            self.dump_tree(output, "", root)?;
         }
         Ok(())
     }
@@ -268,7 +273,7 @@ impl PacketDumper {
         Ok(())
     }
 
-    fn dump_packet(&self, output: &mut dyn io::Write, i: &str,
+    fn dump_packet(&self, mut output: &mut dyn io::Write, i: &str,
                   header: Option<&Header>, p: &Packet, map: Option<&Map>,
                   additional_fields: Option<&Vec<String>>)
                   -> Result<()> {
@@ -350,8 +355,8 @@ impl PacketDumper {
                     },
                     mpi::PublicKey::ECDH { curve, q, hash, sym } => {
                         writeln!(output, "{}  Curve: {}", ii, curve)?;
-                        writeln!(output, "{}  Hash algo: {}", ii, hash)?;
-                        writeln!(output, "{}  Symmetric algo: {}", ii,
+                        writeln!(output, "{}  KDF hash algo: {}", ii, hash)?;
+                        writeln!(output, "{}  KEK symmetric algo: {}", ii,
                                  sym)?;
                         pd.dump_mpis(output, &ii, &[q.value()], &["q"])?;
                     },
@@ -372,7 +377,7 @@ impl PacketDumper {
                     },
 
                     // crypto::mpi:Publickey is non-exhaustive
-                    _ => writeln!(output, "{}  Unknown variant", ii)?,
+                    u => writeln!(output, "{}Unknown variant: {:?}", ii, u)?,
                 }
             }
 
@@ -430,7 +435,7 @@ impl PacketDumper {
                                     },
 
                                     // crypto::mpi::SecretKeyMaterial is non-exhaustive.
-                                    _ => writeln!(output, "{}  Unknown variant", ii)?,
+                                    u => writeln!(output, "{}Unknown variant: {:?}", ii, u)?,
                                 }
                                 Ok(())
                             })?;
@@ -537,7 +542,7 @@ impl PacketDumper {
                         },
 
                         // crypto::mpi::Signature is non-exhaustive.
-                        _ => writeln!(output, "{}  Unknown variant", ii)?,
+                        u => writeln!(output, "{}Unknown variant: {:?}", ii, u)?,
                     }
                 }
             },
@@ -552,7 +557,11 @@ impl PacketDumper {
             },
 
             Trust(ref p) => {
-                writeln!(output, "{}  Value: {}", i, hex::encode(p.value()))?;
+                writeln!(output, "{}  Value:", i)?;
+                let mut hd = hex::Dumper::new(
+                    &mut output,
+                    self.indentation_for_hexdump(&format!("{}  ", i), 16));
+                hd.write_ascii(p.value())?;
             },
 
             UserID(ref u) => {
@@ -648,7 +657,7 @@ impl PacketDumper {
                         },
 
                         // crypto::mpi::Ciphertext is non-exhaustive.
-                        _ => writeln!(output, "{}  Unknown variant", ii)?,
+                        u => writeln!(output, "{}Unknown variant: {:?}", ii, u)?,
                     }
                 }
             },
@@ -687,7 +696,7 @@ impl PacketDumper {
                     },
 
                     // SKESK is non-exhaustive.
-                    _ => writeln!(output, "{}  Unknown variant", i)?,
+                    u => writeln!(output, "{}    Unknown variant: {:?}", i, u)?,
                 }
             },
 
@@ -711,7 +720,7 @@ impl PacketDumper {
             },
 
             // openpgp::Packet is non-exhaustive.
-            _ => writeln!(output, "{}  Unknown variant", i)?,
+            u => writeln!(output, "{}    Unknown variant: {:?}", i, u)?,
         }
 
         if let Some(fields) = additional_fields {
@@ -720,8 +729,9 @@ impl PacketDumper {
             }
         }
 
+        writeln!(output, "{}", i)?;
+
         if let Some(map) = map {
-            writeln!(output, "{}", i)?;
             let mut hd = hex::Dumper::new(output, self.indentation_for_hexdump(
                 i, map.iter()
                     .map(|f| if f.name() == "body" { 16 } else { f.name().len() })
@@ -738,8 +748,6 @@ impl PacketDumper {
 
             let output = hd.into_inner();
             writeln!(output, "{}", i)?;
-        } else {
-            writeln!(output, "{}", i)?;
         }
 
         Ok(())
@@ -770,7 +778,7 @@ impl PacketDumper {
                 write!(output, "{}    Signature expiration time: {} ({})",
                        i, t.convert(),
                        if let Some(creation) = sig.signature_creation_time() {
-                           (creation + t.clone().into()).convert().to_string()
+                           (creation + (*t).into()).convert().to_string()
                        } else {
                            " (no Signature Creation Time subpacket)".into()
                        })?,
@@ -864,9 +872,20 @@ impl PacketDumper {
                        .collect::<Vec<String>>().join(", "))?,
             IntendedRecipient(ref fp) =>
                 write!(output, "{}    Intended Recipient: {}", i, fp)?,
+            AttestedCertifications(digests) => {
+                write!(output, "{}    Attested Certifications:", i)?;
+                if digests.is_empty() {
+                    writeln!(output, " None")?;
+                } else {
+                    writeln!(output)?;
+                    for d in digests {
+                        writeln!(output, "{}      {}", i, hex::encode(d))?;
+                    }
+                }
+            },
 
             // SubpacketValue is non-exhaustive.
-            _ => writeln!(output, "{}  Unknown variant", i)?,
+            u => writeln!(output, "{}    Unknown variant: {:?}", i, u)?,
         }
 
         match s.value() {
@@ -929,7 +948,7 @@ impl PacketDumper {
             },
 
             // S2K is non-exhaustive
-            _ => writeln!(output, "{}  Unknown variant", i)?,
+            u => writeln!(output, "{}    Unknown variant: {:?}", i, u)?,
         }
         Ok(())
     }
@@ -937,7 +956,7 @@ impl PacketDumper {
     fn dump_mpis(&self, output: &mut dyn io::Write, i: &str,
                  chunks: &[&[u8]], keys: &[&str]) -> Result<()> {
         assert_eq!(chunks.len(), keys.len());
-        if chunks.len() == 0 {
+        if chunks.is_empty() {
             return Ok(());
         }
 
