@@ -1,8 +1,6 @@
 use std::convert::TryFrom;
 use std::io::{self, Read};
 
-use clap;
-
 use sequoia_openpgp as openpgp;
 use crate::openpgp::{KeyHandle, Packet, Result};
 use crate::openpgp::cert::prelude::*;
@@ -11,16 +9,21 @@ use openpgp::packet::{
     key::PublicParts,
 };
 use crate::openpgp::parse::{Parse, PacketParserResult};
-use crate::openpgp::policy::Policy;
+use crate::openpgp::policy::{Policy, HashAlgoSecurity};
 use crate::openpgp::packet::key::SecretKeyMaterial;
 
 use super::dump::Convert;
 
-pub fn inspect(m: &clap::ArgMatches, policy: &dyn Policy, output: &mut dyn io::Write)
-               -> Result<()> {
-    let print_certifications = m.is_present("certifications");
+use crate::SECONDS_IN_YEAR;
+use crate::SECONDS_IN_DAY;
 
-    let input = m.value_of("input");
+use crate::sq_cli::InspectCommand;
+
+pub fn inspect(c: InspectCommand, policy: &dyn Policy, output: &mut dyn io::Write)
+               -> Result<()> {
+    let print_certifications = c.certifications;
+
+    let input = c.input.as_deref();
     let input_name = input.unwrap_or("-");
     write!(output, "{}: ", input_name)?;
 
@@ -46,7 +49,7 @@ pub fn inspect(m: &clap::ArgMatches, policy: &dyn Policy, output: &mut dyn io::W
                         type_called = true;
                     }
                     let pp = openpgp::PacketPile::from(
-                        ::std::mem::replace(&mut packets, Vec::new()));
+                        std::mem::take(&mut packets));
                     let cert = openpgp::Cert::try_from(pp)?;
                     inspect_cert(policy, output, &cert,
                                  print_certifications)?;
@@ -142,7 +145,7 @@ fn inspect_cert(policy: &dyn Policy,
     writeln!(output)?;
     writeln!(output, "    Fingerprint: {}", cert.fingerprint())?;
     inspect_revocation(output, "", cert.revocation_status(policy, None))?;
-    inspect_key(policy, output, "", cert.keys().nth(0).unwrap(),
+    inspect_key(policy, output, "", cert.keys().next().unwrap(),
                 print_certifications)?;
     writeln!(output)?;
 
@@ -174,7 +177,7 @@ fn inspect_cert(policy: &dyn Policy,
             }
             Err(e) => print_error_chain(output, &e)?,
         }
-        inspect_certifications(output,
+        inspect_certifications(output, policy,
                                uidb.certifications(),
                                print_certifications)?;
         writeln!(output)?;
@@ -192,7 +195,7 @@ fn inspect_cert(policy: &dyn Policy,
             }
             Err(e) => print_error_chain(output, &e)?,
         }
-        inspect_certifications(output,
+        inspect_certifications(output, policy,
                                uab.certifications(),
                                print_certifications)?;
         writeln!(output)?;
@@ -208,7 +211,7 @@ fn inspect_cert(policy: &dyn Policy,
             }
             Err(e) => print_error_chain(output, &e)?,
         }
-        inspect_certifications(output,
+        inspect_certifications(output, policy,
                                ub.certifications(),
                                print_certifications)?;
         writeln!(output)?;
@@ -271,7 +274,8 @@ fn inspect_key(policy: &dyn Policy,
             writeln!(output, "{}      Key flags: {}", indent, flags)?;
         }
     }
-    inspect_certifications(output, bundle.certifications().iter(),
+    inspect_certifications(output, policy,
+                           bundle.certifications().iter(),
                            print_certifications)?;
 
     Ok(())
@@ -283,14 +287,38 @@ fn inspect_revocation(output: &mut dyn io::Write,
                       -> Result<()> {
     use crate::openpgp::types::RevocationStatus::*;
     fn print_reasons(output: &mut dyn io::Write, indent: &str,
-                     sigs: &[&Signature])
+                     third_party: bool, sigs: &[&Signature])
                      -> Result<()> {
         for sig in sigs {
-            if let Some((r, _)) = sig.reason_for_revocation() {
+            if let Some((r, msg)) = sig.reason_for_revocation() {
                 writeln!(output, "{}                  - {}", indent, r)?;
+                if third_party {
+                    writeln!(output, "{}                    Issued by {}",
+                             indent,
+                             if let Some(issuer)
+                                 = sig.get_issuers().into_iter().next()
+                             {
+                                 issuer.to_string()
+                             } else {
+                                 "an unknown certificate".into()
+                             })?;
+                }
+                writeln!(output, "{}                    Message: {:?}",
+                         indent, String::from_utf8_lossy(msg))?;
             } else {
                 writeln!(output, "{}                  - No reason specified",
                          indent)?;
+                if third_party {
+                    writeln!(output, "{}                    Issued by {}",
+                             indent,
+                             if let Some(issuer)
+                                 = &sig.get_issuers().into_iter().next()
+                             {
+                                 issuer.to_string()
+                             } else {
+                                 "an unknown certificate".into()
+                             })?;
+                }
             }
         }
         Ok(())
@@ -298,11 +326,11 @@ fn inspect_revocation(output: &mut dyn io::Write,
     match revoked {
         Revoked(sigs) => {
             writeln!(output, "{}                 Revoked:", indent)?;
-            print_reasons(output, indent, &sigs)?;
+            print_reasons(output, indent, false, &sigs)?;
         },
         CouldBe(sigs) => {
             writeln!(output, "{}                 Possibly revoked:", indent)?;
-            print_reasons(output, indent, &sigs)?;
+            print_reasons(output, indent, true, &sigs)?;
         },
         NotAsFarAsWeKnow => (),
     }
@@ -334,7 +362,7 @@ fn inspect_key_flags(flags: openpgp::types::KeyFlags) -> Option<String> {
         capabilities.push("split key")
     }
 
-    if capabilities.len() > 0 {
+    if !capabilities.is_empty() {
         Some(capabilities.join(", "))
     } else {
         None
@@ -347,7 +375,7 @@ fn inspect_signatures(output: &mut dyn io::Write,
     for sig in sigs {
         match sig.typ() {
             Binary | Text => (),
-            signature_type @ _ =>
+            signature_type =>
                 writeln!(output, "           Kind: {}", signature_type)?,
         }
 
@@ -376,31 +404,113 @@ fn inspect_signatures(output: &mut dyn io::Write,
 }
 
 fn inspect_certifications<'a, A>(output: &mut dyn io::Write,
-                          certs: A,
-                          print_certifications: bool) -> Result<()> where
-        A: std::iter::Iterator<Item=&'a openpgp::packet::Signature> {
+                                 policy: &dyn Policy,
+                                 certs: A,
+                                 print_certifications: bool)
+    -> Result<()>
+    where A: std::iter::Iterator<Item=&'a openpgp::packet::Signature>
+{
     if print_certifications {
         let mut emit_warning = false;
         for sig in certs {
+            let time = if let Some(time) = sig.signature_creation_time() {
+                chrono::DateTime::<chrono::offset::Utc>::from(time)
+            } else {
+                // A signature must have a signature creation time
+                // subpacket to be valid.  This signature is not
+                // valid, so skip it.
+                continue;
+            };
+
             emit_warning = true;
+
+            writeln!(output, "  Certification: Creation time: {}", time)?;
+
+            let indent = "                 ";
+
+            if let Some(e) = sig.signature_expiration_time() {
+                let e = chrono::DateTime::<chrono::offset::Utc>::from(e);
+                let diff = e - time;
+                let years = diff.num_seconds() / (SECONDS_IN_YEAR as i64);
+                let rest = diff.num_seconds() - years * (SECONDS_IN_YEAR as i64);
+                let days = rest / (SECONDS_IN_DAY as i64);
+                let rest = rest - days * (SECONDS_IN_DAY as i64);
+
+                writeln!(output, "{}Expiration time: {} (after {}{}{}{}{})",
+                         indent,
+                         e,
+                         match years {
+                             0 => "".into(),
+                             1 => format!("{} year", years),
+                             _ => format!("{} years", years),
+                         },
+                         if years != 0 && days != 0 { ", " } else { "" },
+                         match days {
+                             0 => "".into(),
+                             1 => format!("{} day", days),
+                             _ => format!("{} days", days),
+                         },
+                         if years == 0 && days != 0 && rest != 0 { ", " } else { "" },
+                         if years == 0 {
+                             match rest {
+                                 0 => "".into(),
+                                 1 => format!("{} second", rest),
+                                 _ => format!("{} seconds", rest),
+                             }
+                         } else {
+                             "".into()
+                         })?;
+            }
+
+            if let Some((depth, amount)) = sig.trust_signature() {
+                writeln!(output, "{}Trust depth: {}", indent,
+                         depth)?;
+                writeln!(output, "{}Trust amount: {}", indent,
+                         amount)?;
+            }
+            for re in sig.regular_expressions() {
+                if let Ok(re) = String::from_utf8(re.to_vec()) {
+                    writeln!(output, "{}Regular expression: {:?}", indent,
+                             re)?;
+                } else {
+                    writeln!(output,
+                             "{}Regular expression (invalid UTF-8): {:?}",
+                             indent,
+                             String::from_utf8_lossy(re))?;
+                }
+            }
+
             let mut fps: Vec<_> = sig.issuer_fingerprints().collect();
             fps.sort();
             fps.dedup();
             let fps: Vec<KeyHandle> = fps.into_iter().map(|fp| fp.into()).collect();
             for fp in fps.iter() {
-                writeln!(output, "Alleged certifier: {}", fp)?;
+                writeln!(output, "{}Alleged certifier: {}", indent, fp)?;
             }
             let mut keyids: Vec<_> = sig.issuers().collect();
             keyids.sort();
             keyids.dedup();
             for keyid in keyids {
                 if ! fps.iter().any(|fp| fp.aliases(&keyid.into())) {
-                    writeln!(output, "Alleged certifier: {}", keyid)?;
+                    writeln!(output, "{}Alleged certifier: {}", indent,
+                             keyid)?;
                 }
             }
+
+            writeln!(output, "{}Hash algorithm: {}",
+                     indent, sig.hash_algo())?;
+            if let Err(err) = policy.signature(
+                sig, HashAlgoSecurity::CollisionResistance)
+            {
+                writeln!(output,
+                         "{}Certificate is not valid according to \
+                          the current policy:\n\
+                          {}  {}",
+                         indent, indent, err)?;
+            }
         }
         if emit_warning {
-            writeln!(output, "             Note: \
+            writeln!(output, "           Note: \
                               Certifications have NOT been verified!")?;
         }
     } else {
