@@ -1,5 +1,4 @@
 use anyhow::Context as _;
-use clap::ArgMatches;
 use itertools::Itertools;
 use std::time::{SystemTime, Duration};
 
@@ -11,7 +10,7 @@ use crate::openpgp::cert::prelude::*;
 use crate::openpgp::packet::prelude::*;
 use crate::openpgp::packet::signature::subpacket::SubpacketTag;
 use crate::openpgp::parse::Parse;
-use crate::openpgp::policy::Policy;
+use crate::openpgp::policy::{Policy, HashAlgoSecurity};
 use crate::openpgp::serialize::Serialize;
 use crate::openpgp::types::KeyFlags;
 use crate::openpgp::types::SignatureType;
@@ -19,28 +18,40 @@ use crate::openpgp::types::SignatureType;
 use crate::{
     open_or_stdin,
 };
+use crate::commands::get_primary_keys;
 use crate::Config;
 use crate::SECONDS_IN_YEAR;
 use crate::parse_duration;
 use crate::decrypt_key;
 
-pub fn dispatch(config: Config, m: &clap::ArgMatches) -> Result<()> {
-    match m.subcommand() {
-        ("generate", Some(m)) => generate(config, m)?,
-        ("extract-cert", Some(m)) => extract_cert(config, m)?,
-        ("adopt", Some(m)) => adopt(config, m)?,
-        ("attest-certifications", Some(m)) =>
-            attest_certifications(config, m)?,
-        _ => unreachable!(),
-        }
+use crate::sq_cli::KeyCommand;
+use crate::sq_cli::KeyGenerateCommand;
+use crate::sq_cli::KeyPasswordCommand;
+use crate::sq_cli::KeyUseridCommand;
+use crate::sq_cli::KeyUseridAddCommand;
+use crate::sq_cli::KeyUseridStripCommand;
+use crate::sq_cli::KeyExtractCertCommand;
+use crate::sq_cli::KeyAdoptCommand;
+use crate::sq_cli::KeyAttestCertificationsCommand;
+use crate::sq_cli::KeySubcommands::*;
+
+pub fn dispatch(config: Config, command: KeyCommand) -> Result<()> {
+    match command.subcommand {
+        Generate(c) => generate(config, c)?,
+        Password(c) => password(config, c)?,
+        Userid(c) => userid(config, c)?,
+        ExtractCert(c) => extract_cert(config, c)?,
+        Adopt(c) => adopt(config, c)?,
+        AttestCertifications(c) => attest_certifications(config, c)?,
+    }
     Ok(())
 }
 
-fn generate(config: Config, m: &ArgMatches) -> Result<()> {
+fn generate(config: Config, command: KeyGenerateCommand) -> Result<()> {
     let mut builder = CertBuilder::new();
 
     // User ID
-    match m.values_of("userid") {
+    match command.userid {
         Some(uids) => for uid in uids {
             builder = builder.add_userid(uid);
         },
@@ -49,8 +60,13 @@ fn generate(config: Config, m: &ArgMatches) -> Result<()> {
         }
     }
 
+    // Creation time.
+    if let Some(t) = command.creation_time {
+        builder = builder.set_creation_time(SystemTime::from(t.time));
+    };
+
     // Expiration.
-    match (m.value_of("expires"), m.value_of("expires-in")) {
+    match (command.expires, command.expires_in) {
         (None, None) => // Default expiration.
             builder = builder.set_validity_period(
                 Some(Duration::new(3 * SECONDS_IN_YEAR, 0))),
@@ -60,7 +76,7 @@ fn generate(config: Config, m: &ArgMatches) -> Result<()> {
             let now = builder.creation_time()
                 .unwrap_or_else(std::time::SystemTime::now);
             let expiration = SystemTime::from(
-                crate::parse_iso8601(t, chrono::NaiveTime::from_hms(0, 0, 0))?);
+                crate::parse_iso8601(&t, chrono::NaiveTime::from_hms(0, 0, 0))?);
             let validity = expiration.duration_since(now)?;
             builder = builder.set_creation_time(now)
                 .set_validity_period(validity);
@@ -68,31 +84,28 @@ fn generate(config: Config, m: &ArgMatches) -> Result<()> {
         (None, Some(d)) if d == "never" =>
             builder = builder.set_validity_period(None),
         (None, Some(d)) => {
-            let d = parse_duration(d)?;
+            let d = parse_duration(&d)?;
             builder = builder.set_validity_period(Some(d));
         },
         (Some(_), Some(_)) => unreachable!("conflicting args"),
     }
 
     // Cipher Suite
-    match m.value_of("cipher-suite") {
-        Some("rsa3k") => {
+    use crate::sq_cli::KeyCipherSuite::*;
+    match command.cipher_suite {
+        Rsa3k => {
             builder = builder.set_cipher_suite(CipherSuite::RSA3k);
         }
-        Some("rsa4k") => {
+        Rsa4k => {
             builder = builder.set_cipher_suite(CipherSuite::RSA4k);
         }
-        Some("cv25519") => {
+        Cv25519 => {
             builder = builder.set_cipher_suite(CipherSuite::Cv25519);
         }
-        Some(ref cs) => {
-            return Err(anyhow::anyhow!("Unknown cipher suite '{}'", cs));
-        }
-        None => panic!("argument has a default value"),
     }
 
     // Signing Capability
-    match (m.is_present("can-sign"), m.is_present("cannot-sign")) {
+    match (command.can_sign, command.cannot_sign) {
         (false, false) | (true, false) => {
             builder = builder.add_signing_subkey();
         }
@@ -103,19 +116,33 @@ fn generate(config: Config, m: &ArgMatches) -> Result<()> {
         }
     }
 
+    // Authentication Capability
+    match (command.can_authenticate, command.cannot_authenticate) {
+        (false, false) | (true, false) => {
+            builder = builder.add_authentication_subkey()
+        }
+        (false, true) => { /* no authentication subkey */ }
+        (true, true) => {
+            return Err(
+                anyhow::anyhow!("Conflicting arguments --can-authenticate and\
+                                --cannot-authenticate"));
+        }
+    }
+
     // Encryption Capability
-    match (m.value_of("can-encrypt"), m.is_present("cannot-encrypt")) {
-        (Some("universal"), false) | (None, false) => {
+    use crate::sq_cli::KeyEncryptPurpose::*;
+    match (command.can_encrypt, command.cannot_encrypt) {
+        (Some(Universal), false) | (None, false) => {
             builder = builder.add_subkey(KeyFlags::empty()
                                          .set_transport_encryption()
                                          .set_storage_encryption(),
                                          None,
                                          None);
         }
-        (Some("storage"), false) => {
+        (Some(Storage), false) => {
             builder = builder.add_storage_encryption_subkey();
         }
-        (Some("transport"), false) => {
+        (Some(Transport), false) => {
             builder = builder.add_transport_encryption_subkey();
         }
         (None, true) => { /* no encryption subkey */ }
@@ -124,13 +151,9 @@ fn generate(config: Config, m: &ArgMatches) -> Result<()> {
                 anyhow::anyhow!("Conflicting arguments --can-encrypt and \
                              --cannot-encrypt"));
         }
-        (Some(ref cap), false) => {
-            return Err(
-                anyhow::anyhow!("Unknown encryption capability '{}'", cap));
-        }
     }
 
-    if m.is_present("with-password") {
+    if command.with_password {
         let p0 = rpassword::read_password_from_tty(Some(
             "Enter password to protect the key: "))?.into();
         let p1 = rpassword::read_password_from_tty(Some(
@@ -147,9 +170,9 @@ fn generate(config: Config, m: &ArgMatches) -> Result<()> {
     let (cert, rev) = builder.generate()?;
 
     // Export
-    if m.is_present("export") {
+    if command.export.is_some() {
         let (key_path, rev_path) =
-            match (m.value_of("export"), m.value_of("rev-cert")) {
+            match (command.export.as_deref(), command.rev_cert.as_deref()) {
                 (Some("-"), Some("-")) =>
                     ("-".to_string(), "-".to_string()),
                 (Some("-"), Some(ref rp)) =>
@@ -205,12 +228,80 @@ fn generate(config: Config, m: &ArgMatches) -> Result<()> {
     Ok(())
 }
 
-fn extract_cert(config: Config, m: &ArgMatches) -> Result<()> {
-    let input = open_or_stdin(m.value_of("input"))?;
-    let mut output = config.create_or_stdout_safe(m.value_of("output"))?;
+fn password(config: Config, command: KeyPasswordCommand) -> Result<()> {
+    let input = open_or_stdin(command.io.input.as_deref())?;
+    let key = Cert::from_reader(input)?;
+
+    if ! key.is_tsk() {
+        return Err(anyhow::anyhow!("Certificate has no secrets"));
+    }
+
+    // First, decrypt all secrets.
+    let passwords = &mut Vec::new();
+    let mut decrypted: Vec<Packet> = vec![decrypt_key(
+        key.primary_key().key().clone().parts_into_secret()?,
+        passwords,
+    )?
+    .into()];
+    for ka in key.keys().subkeys().secret() {
+        decrypted.push(decrypt_key(
+            ka.key().clone().parts_into_secret()?,
+            passwords)?.into());
+    }
+    let mut key = key.insert_packets(decrypted)?;
+    assert_eq!(key.keys().secret().count(),
+               key.keys().unencrypted_secret().count());
+
+    let new_password = if command.clear {
+        None
+    } else {
+        let prompt_0 =
+            rpassword::read_password_from_tty(Some("New password: "))
+            .context("Error reading password")?;
+        let prompt_1 =
+            rpassword::read_password_from_tty(Some("Repeat new password: "))
+            .context("Error reading password")?;
+
+        if prompt_0 != prompt_1 {
+            return Err(anyhow::anyhow!("Passwords do not match"));
+        }
+
+        if prompt_0.is_empty() {
+            // Empty password means no password.
+            None
+        } else {
+            Some(prompt_0.into())
+        }
+    };
+
+    if let Some(new) = new_password {
+        let mut encrypted: Vec<Packet> = vec![
+            key.primary_key().key().clone().parts_into_secret()?
+                .encrypt_secret(&new)?.into()
+        ];
+        for ka in key.keys().subkeys().unencrypted_secret() {
+            encrypted.push(
+                ka.key().clone().parts_into_secret()?
+                    .encrypt_secret(&new)?.into());
+        }
+        key = key.insert_packets(encrypted)?;
+    }
+
+    let mut output = config.create_or_stdout_safe(command.io.output.as_deref())?;
+    if command.binary {
+        key.as_tsk().serialize(&mut output)?;
+    } else {
+        key.as_tsk().armored().serialize(&mut output)?;
+    }
+    Ok(())
+}
+
+fn extract_cert(config: Config, command: KeyExtractCertCommand) -> Result<()> {
+    let input = open_or_stdin(command.io.input.as_deref())?;
+    let mut output = config.create_or_stdout_safe(command.io.output.as_deref())?;
 
     let cert = Cert::from_reader(input)?;
-    if m.is_present("binary") {
+    if command.binary {
         cert.serialize(&mut output)?;
     } else {
         cert.armored().serialize(&mut output)?;
@@ -218,8 +309,210 @@ fn extract_cert(config: Config, m: &ArgMatches) -> Result<()> {
     Ok(())
 }
 
-fn adopt(config: Config, m: &ArgMatches) -> Result<()> {
-    let input = open_or_stdin(m.value_of("certificate"))?;
+fn userid(config: Config, command: KeyUseridCommand) -> Result<()> {
+    match command {
+        KeyUseridCommand::Add(c) => userid_add(config, c)?,
+        KeyUseridCommand::Strip(c) => userid_strip(config, c)?,
+    }
+
+    Ok(())
+}
+
+fn userid_add(config: Config, command: KeyUseridAddCommand) -> Result<()> {
+    let input = open_or_stdin(command.io.input.as_deref())?;
+    let key = Cert::from_reader(input)?;
+
+    // Fail if any of the User IDs to add already exist in the ValidCert
+    let key_userids: Vec<_> =
+        key.userids().map(|u| u.userid().value()).collect();
+    let exists: Vec<_> =  command.userid.iter()
+        .filter(|s| key_userids.contains(&s.as_bytes()))
+        .collect();
+    if ! exists.is_empty() {
+        return Err(anyhow::anyhow!(
+            "The certificate already contains the User ID(s) {}.",
+            exists.iter().map(|s| format!("{:?}", s)).join(", ")));
+    }
+
+    let creation_time =
+        command.creation_time.map(|t| SystemTime::from(t.time));
+
+    // If a password is needed to use the key, the user will be prompted.
+    let pk = get_primary_keys(&[key.clone()], &config.policy,
+                              command.private_key_store.as_deref(),
+                              creation_time, None)?;
+
+    assert_eq!(pk.len(), 1, "Expect exactly one result from get_primary_keys()");
+    let mut pk = pk.into_iter().next().unwrap();
+
+    let vcert = key.with_policy(&config.policy, creation_time)
+        .with_context(|| format!("Certificate {} is not valid",
+                                 key.fingerprint()))?;
+
+    // Use the primary User ID or direct key signature as template for the
+    // SignatureBuilder.
+    //
+    // XXX: Long term, this functionality belongs next to
+    // openpgp/src/cert/builder/key.rs.
+    let mut sb =
+        if let Ok(primary_user_id) = vcert.primary_userid() {
+            SignatureBuilder::from(primary_user_id.binding_signature().clone())
+        } else if let Ok(direct_key_sig) = vcert.direct_key_signature() {
+            SignatureBuilder::from(direct_key_sig.clone())
+                .set_type(SignatureType::PositiveCertification)
+        } else {
+            // If there is neither a valid uid binding signature, nor a
+            // valid direct key signature, we shouldn't have gotten a
+            // ValidCert above.
+            unreachable!("ValidCert has to have one of the above.")
+        };
+
+    // Remove bad algorithms from preferred algorithm subpackets,
+    // and make sure preference lists contain at least one good algorithm.
+
+    // - symmetric_algorithms
+    let mut symmetric_algorithms: Vec<_> =
+        sb.preferred_symmetric_algorithms().unwrap_or(&[]).to_vec();
+    symmetric_algorithms
+        .retain(|algo| config.policy.symmetric_algorithm(*algo).is_ok());
+    if symmetric_algorithms.is_empty() {
+        symmetric_algorithms.push(Default::default());
+    }
+    sb = sb.set_preferred_symmetric_algorithms(symmetric_algorithms)?;
+
+    // - hash_algorithms
+    let mut hash_algorithms: Vec<_> =
+        sb.preferred_hash_algorithms().unwrap_or(&[]).to_vec();
+    hash_algorithms.retain(|algo|
+        config.policy
+            .hash_cutoff(*algo, HashAlgoSecurity::CollisionResistance)
+            .map(|cutoff| cutoff.lt(&SystemTime::now()))
+            .unwrap_or(true)
+    );
+    if hash_algorithms.is_empty() {
+        hash_algorithms.push(Default::default());
+    }
+    sb = sb.set_preferred_hash_algorithms(hash_algorithms)?;
+
+
+    // Remove the following types of SubPacket, if they exist
+    const REMOVE_SUBPACKETS: &[SubpacketTag] = &[
+        // The Signature should be exportable.
+        // https://openpgp-wg.gitlab.io/rfc4880bis/#name-exportable-certification
+        // "If this packet is not present, the certification is exportable;
+        // it is equivalent to a flag containing a 1."
+        SubpacketTag::ExportableCertification,
+
+        // PreferredAEADAlgorithms has been removed by WG.
+        // It was replaced by `39  Preferred AEAD Ciphersuites`,
+        //  see https://openpgp-wg.gitlab.io/rfc4880bis/#section-5.2.3.5-7)
+        SubpacketTag::PreferredAEADAlgorithms,
+
+        // Strip the primary userid SubPacket
+        // (don't implicitly make a User ID primary)
+        SubpacketTag::PrimaryUserID,
+
+        // Other SubPacket types that shouldn't be in use in this context
+        SubpacketTag::TrustSignature,
+        SubpacketTag::RegularExpression,
+        SubpacketTag::SignersUserID,
+        SubpacketTag::ReasonForRevocation,
+        SubpacketTag::SignatureTarget,
+        SubpacketTag::EmbeddedSignature,
+        SubpacketTag::AttestedCertifications,
+    ];
+
+    sb = sb.modify_hashed_area(|mut subpacket_area| {
+        REMOVE_SUBPACKETS.iter()
+            .for_each(|sp| subpacket_area.remove_all(*sp));
+
+        Ok(subpacket_area)
+    })?;
+
+    // New User ID should only be made primary if explicitly specified by user.
+    // xxx: add a parameter to set a new user id as primary?
+
+
+    // Collect packets to add to the key (new User IDs and binding signatures)
+    let mut add: Vec<Packet> = vec![];
+
+    // Make new User IDs and binding signatures
+    for uid in command.userid {
+        let uid: UserID = uid.into();
+        add.push(uid.clone().into());
+
+        // Creation time.
+        if let Some(t) = creation_time {
+            sb = sb.set_signature_creation_time(t)?;
+        };
+
+        let binding = uid.bind(&mut pk, &key, sb.clone())?;
+        add.push(binding.into());
+    }
+
+    // Merge additional User IDs into key
+    let cert = key.insert_packets(add)?;
+
+    let mut sink = config.create_or_stdout_safe(command.io.output.as_deref())?;
+    if command.binary {
+        cert.as_tsk().serialize(&mut sink)?;
+    } else {
+        cert.as_tsk().armored().serialize(&mut sink)?;
+    }
+    Ok(())
+}
+
+fn userid_strip(config: Config, command: KeyUseridStripCommand) -> Result<()> {
+    let input = open_or_stdin(command.io.input.as_deref())?;
+    let key = Cert::from_reader(input)?;
+
+    let orig_cert_valid = key.with_policy(&config.policy, None).is_ok();
+
+    let strip: Vec<_> = command.userid;
+
+    // Make sure that each User ID that the user requested to remove exists in
+    // `key`, and *can* be removed.
+    let key_userids: Vec<_> =
+        key.userids().map(|u| u.userid().value()).collect();
+
+    let missing: Vec<_> = strip.iter()
+        .filter(|s| !key_userids.contains(&s.as_bytes()))
+        .collect();
+    if ! missing.is_empty() {
+        return Err(anyhow::anyhow!(
+            "The certificate doesn't contain the User ID(s) {}.",
+            missing.iter().map(|s| format!("{:?}", s)).join(", ")));
+    }
+
+    let cert = key.retain_userids(|uid| {
+        // Don't keep User IDs that were selected for removal
+        ! strip.iter().any(|rm| rm.as_bytes() == uid.userid().value())
+    });
+
+    if orig_cert_valid {
+        if let Err(err) = cert.with_policy(&config.policy, None) {
+            eprintln!(
+"Removing the User ID(s) has resulted in a invalid key:
+{}
+
+You could create a direct key signature or update the self
+signatures on other User IDs to make the key valid again.",
+                err
+            );
+        }
+    }
+
+    let mut sink = config.create_or_stdout_safe(command.io.output.as_deref())?;
+    if command.binary {
+        cert.as_tsk().serialize(&mut sink)?;
+    } else {
+        cert.as_tsk().armored().serialize(&mut sink)?;
+    }
+    Ok(())
+}
+
+fn adopt(config: Config, command: KeyAdoptCommand) -> Result<()> {
+    let input = open_or_stdin(command.certificate.as_deref())?;
     let cert = Cert::from_reader(input)?;
     let mut wanted: Vec<(KeyHandle,
                          Option<(Key<key::PublicParts, key::SubordinateRole>,
@@ -227,7 +520,7 @@ fn adopt(config: Config, m: &ArgMatches) -> Result<()> {
         = vec![];
 
     // Gather the Key IDs / Fingerprints and make sure they are valid.
-    for id in m.values_of("key").unwrap_or_default() {
+    for id in command.key {
         let h = id.parse::<KeyHandle>()?;
         if h.is_invalid() {
             return Err(anyhow::anyhow!(
@@ -238,16 +531,16 @@ fn adopt(config: Config, m: &ArgMatches) -> Result<()> {
 
     let null_policy = &crate::openpgp::policy::NullPolicy::new();
     let adoptee_policy: &dyn Policy =
-        if m.values_of("allow-broken-crypto").is_some() {
+        if command.allow_broken_crypto {
             null_policy
         } else {
             &config.policy
         };
 
     // Find the corresponding keys.
-    for keyring in m.values_of("keyring").unwrap_or_default() {
-        for cert in CertParser::from_file(keyring)
-            .context(format!("Parsing: {}", keyring))?
+    for keyring in command.keyring {
+        for cert in CertParser::from_file(&keyring)
+            .context(format!("Parsing: {}", &keyring))?
         {
             let cert = cert.context(format!("Parsing {}", keyring))?;
 
@@ -314,7 +607,7 @@ fn adopt(config: Config, m: &ArgMatches) -> Result<()> {
             }
         })
         .collect();
-    if missing.len() > 0 {
+    if !missing.is_empty() {
         return Err(anyhow::anyhow!(
             "Keys not found: {}",
             missing.iter().map(|&h| h.to_hex()).join(", ")));
@@ -352,12 +645,11 @@ fn adopt(config: Config, m: &ArgMatches) -> Result<()> {
                 .into_keypair()?;
 
             let backsig = builder.embedded_signatures()
-                .filter(|backsig| {
+                .find(|backsig| {
                     (*backsig).clone().verify_primary_key_binding(
                         &cert.primary_key(),
                         &key).is_ok()
                 })
-                .nth(0)
                 .map(|sig| SignatureBuilder::from(sig.clone()))
                 .unwrap_or_else(|| {
                     SignatureBuilder::new(SignatureType::PrimaryKeyBinding)
@@ -384,8 +676,8 @@ fn adopt(config: Config, m: &ArgMatches) -> Result<()> {
 
     let cert = cert.clone().insert_packets(packets.clone())?;
 
-    let mut sink = config.create_or_stdout_safe(m.value_of("output"))?;
-    if m.is_present("binary") {
+    let mut sink = config.create_or_stdout_safe(command.output.as_deref())?;
+    if command.binary {
         cert.as_tsk().serialize(&mut sink)?;
     } else {
         cert.as_tsk().armored().serialize(&mut sink)?;
@@ -422,44 +714,14 @@ fn adopt(config: Config, m: &ArgMatches) -> Result<()> {
     Ok(())
 }
 
-fn attest_certifications(config: Config, m: &ArgMatches)
+fn attest_certifications(config: Config, command: KeyAttestCertificationsCommand)
                          -> Result<()> {
-    // XXX: This function has to do some steps manually, because
-    // Sequoia does not expose this functionality because it has not
-    // been standardized yet.
-    use sequoia_openpgp::{
-        crypto::hash::{Hash, Digest},
-        packet::signature::subpacket::*,
-        types::HashAlgorithm,
-    };
-    #[allow(non_upper_case_globals)]
-    const SignatureType__AttestedKey: SignatureType =
-        SignatureType::Unknown(0x16);
-    #[allow(non_upper_case_globals)]
-    const SubpacketTag__AttestedCertifications: SubpacketTag =
-        SubpacketTag::Unknown(37);
-
     // Attest to all certifications?
-    let all = ! m.is_present("none"); // All is the default.
+    let all = !command.none; // All is the default.
 
-    // Some configuration.
-    let hash_algo = HashAlgorithm::default();
-    let digest_size = hash_algo.context()?.digest_size();
-    let reserve_area_space = 256; // For the other subpackets.
-    let digests_per_sig = ((1usize << 16) - reserve_area_space) / digest_size;
-
-    let input = open_or_stdin(m.value_of("key"))?;
+    let input = open_or_stdin(command.key.as_deref())?;
     let key = Cert::from_reader(input)?;
 
-    // First, remove all attestations.
-    let key = Cert::from_packets(
-        key.into_packets().filter(|p| match p {
-            Packet::Signature(s) if s.typ() == SignatureType__AttestedKey =>
-                false,
-            _ => true,
-        }))?;
-
-
     // Get a signer.
     let mut passwords = Vec::new();
     let pk = key.primary_key().key();
@@ -472,96 +734,36 @@ fn attest_certifications(config: Config, m: &ArgMatches)
     // Now, create new attestation signatures.
     let mut attestation_signatures = Vec::new();
     for uid in key.userids() {
-        let mut attestations = Vec::new();
-
         if all {
-            for certification in uid.certifications() {
-                let mut h = hash_algo.context()?;
-                certification.hash_for_confirmation(&mut h);
-                attestations.push(h.into_digest()?);
-            }
-        }
-
-        // Hashes SHOULD be sorted.
-        attestations.sort();
-
-        // All attestation signatures we generate for this component
-        // should have the same creation time.  Fix it now.
-        let t = std::time::SystemTime::now();
-
-        // Hash the components like in a binding signature.
-        let mut hash = hash_algo.context()?;
-        key.primary_key().hash(&mut hash);
-        uid.hash(&mut hash);
-
-        for digests in attestations.chunks(digests_per_sig) {
-            let mut body = Vec::with_capacity(digest_size * digests.len());
-            digests.iter().for_each(|d| body.extend(d));
-
-            attestation_signatures.push(
-                SignatureBuilder::new(SignatureType__AttestedKey)
-                    .set_signature_creation_time(t)?
-                    .modify_hashed_area(|mut a| {
-                        a.add(Subpacket::new(
-                            SubpacketValue::Unknown {
-                                tag: SubpacketTag__AttestedCertifications,
-                                body,
-                            },
-                            true)?)?;
-                        Ok(a)
-                    })?
-                    .sign_hash(&mut pk_signer, hash.clone())?);
+            attestation_signatures.append(
+                &mut uid.attest_certifications(&config.policy,
+                                               &mut pk_signer,
+                                               uid.certifications())?);
+        } else {
+            attestation_signatures.append(
+                &mut uid.attest_certifications(&config.policy,
+                                               &mut pk_signer, &[])?);
         }
     }
 
     for ua in key.user_attributes() {
-        let mut attestations = Vec::new();
-
         if all {
-            for certification in ua.certifications() {
-                let mut h = hash_algo.context()?;
-                certification.hash_for_confirmation(&mut h);
-                attestations.push(h.into_digest()?);
-            }
-        }
-
-        // Hashes SHOULD be sorted.
-        attestations.sort();
-
-        // All attestation signatures we generate for this component
-        // should have the same creation time.  Fix it now.
-        let t = std::time::SystemTime::now();
-
-        // Hash the components like in a binding signature.
-        let mut hash = hash_algo.context()?;
-        key.primary_key().hash(&mut hash);
-        ua.hash(&mut hash);
-
-        for digests in attestations.chunks(digests_per_sig) {
-            let mut body = Vec::with_capacity(digest_size * digests.len());
-            digests.iter().for_each(|d| body.extend(d));
-
-            attestation_signatures.push(
-                SignatureBuilder::new(SignatureType__AttestedKey)
-                    .set_signature_creation_time(t)?
-                    .modify_hashed_area(|mut a| {
-                        a.add(Subpacket::new(
-                            SubpacketValue::Unknown {
-                                tag: SubpacketTag__AttestedCertifications,
-                                body,
-                            },
-                            true)?)?;
-                        Ok(a)
-                    })?
-                    .sign_hash(&mut pk_signer, hash.clone())?);
+            attestation_signatures.append(
+                &mut ua.attest_certifications(&config.policy,
+                                              &mut pk_signer,
+                                              ua.certifications())?);
+        } else {
+            attestation_signatures.append(
+                &mut ua.attest_certifications(&config.policy,
+                                              &mut pk_signer, &[])?);
         }
     }
 
     // Finally, add the new signatures.
     let key = key.insert_packets(attestation_signatures)?;
 
-    let mut sink = config.create_or_stdout_safe(m.value_of("output"))?;
-    if m.is_present("binary") {
+    let mut sink = config.create_or_stdout_safe(command.output.as_deref())?;
+    if command.binary {
         key.as_tsk().serialize(&mut sink)?;
     } else {
         key.as_tsk().armored().serialize(&mut sink)?;
