@@ -1,11 +1,12 @@
 use anyhow::Context as _;
+use std::borrow::Borrow;
 use std::cmp::Ordering;
 use std::collections::{HashMap, HashSet};
 use std::fs::File;
 use std::io::{self, Write};
 use std::time::SystemTime;
-use rpassword;
 
+use sequoia_net::pks;
 use sequoia_openpgp as openpgp;
 use crate::openpgp::{
     armor,
@@ -28,18 +29,24 @@ use crate::openpgp::serialize::stream::{
     padding::Padder,
 };
 use crate::openpgp::policy::Policy;
+use crate::openpgp::types::KeyFlags;
+use crate::openpgp::types::RevocationStatus;
 
 use crate::{
     Config,
-    parse_armor_kind,
 };
 
+use crate::sq_cli::EncryptCompressionMode;
+use crate::sq_cli::EncryptEncryptionMode;
+use crate::sq_cli::PacketJoinCommand;
+
 #[cfg(feature = "autocrypt")]
 pub mod autocrypt;
 pub mod decrypt;
 pub use self::decrypt::decrypt;
 pub mod sign;
 pub use self::sign::sign;
+pub mod revoke;
 pub mod dump;
 pub use self::dump::dump;
 mod inspect;
@@ -48,79 +55,307 @@ pub mod key;
 pub mod merge_signatures;
 pub use self::merge_signatures::merge_signatures;
 pub mod keyring;
-#[cfg(feature = "net")]
 pub mod net;
 pub mod certify;
 
+#[derive(Debug, Clone, PartialEq, Eq)]
+pub enum GetKeysOptions {
+    AllowNotAlive,
+    AllowRevoked,
+}
+
+enum KeyType {
+    Primary,
+    KeyFlags(KeyFlags),
+}
+
 /// Returns suitable signing keys from a given list of Certs.
-fn get_signing_keys(certs: &[openpgp::Cert], p: &dyn Policy,
-                    timestamp: Option<SystemTime>)
-    -> Result<Vec<crypto::KeyPair>>
+fn get_keys<C>(certs: &[C], p: &dyn Policy,
+               private_key_store: Option<&str>,
+               timestamp: Option<SystemTime>,
+               keytype: KeyType,
+               options: Option<&[GetKeysOptions]>)
+    -> Result<Vec<Box<dyn crypto::Signer + Send + Sync>>>
+    where C: Borrow<Cert>
 {
-    let mut keys = Vec::new();
+    let mut bad = Vec::new();
+
+    let options = options.unwrap_or(&[][..]);
+    let allow_not_alive = options.contains(&GetKeysOptions::AllowNotAlive);
+    let allow_revoked = options.contains(&GetKeysOptions::AllowRevoked);
+
+    let mut keys: Vec<Box<dyn crypto::Signer + Send + Sync>> = Vec::new();
     'next_cert: for tsk in certs {
-        for key in tsk.keys().with_policy(p, timestamp).alive().revoked(false)
-            .for_signing()
-            .supported()
-            .map(|ka| ka.key())
-        {
+        let tsk = tsk.borrow();
+        let vc = match tsk.with_policy(p, timestamp) {
+            Ok(vc) => vc,
+            Err(err) => {
+                return Err(
+                    err.context(format!("Found no suitable key on {}", tsk)));
+            }
+        };
+
+        let keyiter = match keytype {
+            KeyType::Primary => {
+                Box::new(
+                    std::iter::once(
+                        vc.keys()
+                            .next()
+                            .expect("a valid cert has a primary key")))
+                    as Box<dyn Iterator<Item=ValidErasedKeyAmalgamation<openpgp::packet::key::PublicParts>>>
+            },
+            KeyType::KeyFlags(ref flags) => {
+                Box::new(vc.keys().key_flags(flags.clone()))
+                    as Box<dyn Iterator<Item=_>>
+            },
+        };
+        for ka in keyiter {
+            let bad_ = [
+                ! allow_not_alive && matches!(ka.alive(), Err(_)),
+                ! allow_revoked && matches!(ka.revocation_status(),
+                                            RevocationStatus::Revoked(_)),
+                ! ka.pk_algo().is_supported(),
+            ];
+            if bad_.iter().any(|x| *x) {
+                bad.push((ka.fingerprint(), bad_));
+                continue;
+            }
+
+            let key = ka.key();
+
             if let Some(secret) = key.optional_secret() {
                 let unencrypted = match secret {
                     SecretKeyMaterial::Encrypted(ref e) => {
                         let password = rpassword::read_password_from_tty(Some(
                             &format!("Please enter password to decrypt {}/{}: ",
-                                     tsk, key))).unwrap();
+                                     tsk, key)))
+                            .context("Reading password from tty")?;
                         e.decrypt(key.pk_algo(), &password.into())
                             .expect("decryption failed")
                     },
                     SecretKeyMaterial::Unencrypted(ref u) => u.clone(),
                 };
 
-                keys.push(crypto::KeyPair::new(key.clone(), unencrypted)
-                          .unwrap());
-                break 'next_cert;
+                keys.push(Box::new(crypto::KeyPair::new(key.clone(), unencrypted)
+                          .unwrap()));
+                continue 'next_cert;
+            } else if let Some(private_key_store) = private_key_store {
+                let password = rpassword::read_password_from_tty(
+                    Some(&format!("Please enter password to key {}/{}: ", tsk, key))).unwrap().into();
+                match pks::unlock_signer(private_key_store, key.clone(), &password) {
+                    Ok(signer) => {
+                        keys.push(signer);
+                        continue 'next_cert;
+                    },
+                    Err(error) => eprintln!("Could not unlock key: {:?}", error),
+                }
             }
         }
 
-        return Err(anyhow::anyhow!(
-            format!("Found no suitable signing key on {}", tsk)));
+        let timestamp = timestamp.map(|t| {
+            chrono::DateTime::<chrono::offset::Utc>::from(t)
+        });
+
+        let mut context = Vec::new();
+        for (fpr, [not_alive, revoked, not_supported]) in bad {
+            let id: String = if fpr == tsk.fingerprint() {
+                fpr.to_string()
+            } else {
+                format!("{}/{}", tsk.fingerprint(), fpr)
+            };
+
+            let preface = if let Some(t) = timestamp {
+                format!("{} was not considered because\n\
+                         at the specified time ({}) it was",
+                        id, t)
+            } else {
+                format!("{} was not considered because\nit is", fpr)
+            };
+
+            let mut reasons = Vec::new();
+            if not_alive {
+                reasons.push("not alive");
+            }
+            if revoked {
+                reasons.push("revoked");
+            }
+            if not_supported {
+                reasons.push("not supported");
+            }
+
+            context.push(format!("{}: {}",
+                                 preface, reasons.join(", ")));
+        }
+
+        if context.is_empty() {
+            return Err(anyhow::anyhow!(
+                format!("Found no suitable key on {}", tsk)));
+        } else {
+            let context = context.join("\n");
+            return Err(
+                anyhow::anyhow!(
+                    format!("Found no suitable key on {}", tsk))
+                    .context(context));
+        }
     }
 
     Ok(keys)
 }
 
-pub fn encrypt<'a>(policy: &'a dyn Policy,
-                   input: &mut dyn io::Read, message: Message<'a>,
-                   npasswords: usize, recipients: &'a [openpgp::Cert],
-                   signers: Vec<openpgp::Cert>,
-                   mode: openpgp::types::KeyFlags, compression: &str,
-                   time: Option<SystemTime>,
-                   use_expired_subkey: bool,
-)
-                   -> Result<()> {
-    let mut passwords: Vec<crypto::Password> = Vec::with_capacity(npasswords);
-    for n in 0..npasswords {
+/// Returns the primary keys from a given list of Certs.
+///
+/// This returns one key for each Cert.  If a Cert doesn't have an
+/// appropriate key, then this returns an error.
+fn get_primary_keys<C>(certs: &[C], p: &dyn Policy,
+                       private_key_store: Option<&str>,
+                       timestamp: Option<SystemTime>,
+                       options: Option<&[GetKeysOptions]>)
+    -> Result<Vec<Box<dyn crypto::Signer + Send + Sync>>>
+    where C: std::borrow::Borrow<Cert>
+{
+    get_keys(certs, p, private_key_store, timestamp,
+             KeyType::Primary, options)
+}
+
+/// Returns suitable signing keys from a given list of Certs.
+///
+/// This returns one key for each Cert.  If a Cert doesn't have an
+/// appropriate key, then this returns an error.
+fn get_signing_keys<C>(certs: &[C], p: &dyn Policy,
+                       private_key_store: Option<&str>,
+                       timestamp: Option<SystemTime>,
+                       options: Option<&[GetKeysOptions]>)
+    -> Result<Vec<Box<dyn crypto::Signer + Send + Sync>>>
+    where C: Borrow<Cert>
+{
+    get_keys(certs, p, private_key_store, timestamp,
+             KeyType::KeyFlags(KeyFlags::empty().set_signing()),
+             options)
+}
+
+/// Returns suitable certification keys from a given list of Certs.
+///
+/// This returns one key for each Cert.  If a Cert doesn't have an
+/// appropriate key, then this returns an error.
+fn get_certification_keys<C>(certs: &[C], p: &dyn Policy,
+                             private_key_store: Option<&str>,
+                             timestamp: Option<SystemTime>,
+                             options: Option<&[GetKeysOptions]>)
+    -> Result<Vec<Box<dyn crypto::Signer + Send + Sync>>>
+    where C: std::borrow::Borrow<Cert>
+{
+    get_keys(certs, p, private_key_store, timestamp,
+             KeyType::KeyFlags(KeyFlags::empty().set_certification()),
+             options)
+}
+
+// Returns the smallest valid certificate.
+//
+// Given a certificate, returns the smallest valid certificate that is
+// still technically valid according to RFC 4880 and popular OpenPGP
+// implementations.
+//
+// In particular, this function extracts the primary key, and a User
+// ID with its active binding signature.  If there is no valid User
+// ID, it returns the active direct key signature.  If no User ID is
+// specified, or the specified User ID does not occur, then the
+// primary User ID is used and the specified User ID is added without
+// a binding signature.
+pub fn cert_stub(cert: Cert,
+                 policy: &dyn Policy,
+                 timestamp: Option<SystemTime>,
+                 userid: Option<&UserID>)
+    -> Result<Cert>
+{
+    let vc = cert.with_policy(policy, timestamp)?;
+
+    let mut packets = Vec::with_capacity(4);
+    packets.push(Packet::from(vc.primary_key().key().clone()));
+
+    let mut found = false;
+    if let Some(userid) = userid {
+        for u in vc.userids() {
+            if u.userid() == userid {
+                found = true;
+                packets.push(Packet::from(userid.clone()));
+                packets.push(Packet::from(u.binding_signature().clone()));
+            }
+        }
+    }
+    if ! found {
+        // We didn't find the required User ID or no User ID was
+        // specified.  Emit the primary User ID.  If there is none,
+        // emit the direct key signature.
+        if let Ok(uid) = vc.primary_userid() {
+            packets.push(Packet::from(uid.userid().clone()));
+            packets.push(Packet::from(uid.binding_signature().clone()));
+        } else {
+            packets.push(
+                Packet::from(vc.primary_key().binding_signature().clone()));
+        }
+
+        // And include the specified User ID as the very last packet.
+        // This is convenient when we append a revocation certificate
+        // as the revocation certificate is at the right place.
+        if let Some(userid) = userid {
+            packets.push(Packet::from(userid.clone()));
+        }
+    }
+
+    Ok(Cert::from_packets(packets.into_iter())?)
+}
+
+pub struct EncryptOpts<'a> {
+    pub policy: &'a dyn Policy,
+    pub private_key_store: Option<&'a str>,
+    pub input: &'a mut dyn io::Read,
+    pub message: Message<'a>,
+    pub npasswords: usize,
+    pub recipients: &'a [openpgp::Cert],
+    pub signers: Vec<openpgp::Cert>,
+    pub mode: EncryptEncryptionMode,
+    pub compression: EncryptCompressionMode,
+    pub time: Option<SystemTime>,
+    pub use_expired_subkey: bool,
+}
+
+pub fn encrypt(opts: EncryptOpts) -> Result<()> {
+    let mut passwords: Vec<crypto::Password> = Vec::with_capacity(opts.npasswords);
+    for n in 0..opts.npasswords {
         let nprompt = format!("Enter password {}: ", n + 1);
         passwords.push(rpassword::read_password_from_tty(Some(
-            if npasswords > 1 {
+            if opts.npasswords > 1 {
                 &nprompt
             } else {
                 "Enter password: "
             }))?.into());
     }
 
-    if recipients.len() + passwords.len() == 0 {
+    if opts.recipients.len() + passwords.len() == 0 {
         return Err(anyhow::anyhow!(
             "Neither recipient nor password given"));
     }
 
-    let mut signers = get_signing_keys(&signers, policy, time)?;
+    let mode = match opts.mode {
+        EncryptEncryptionMode::Rest => {
+            KeyFlags::empty().set_storage_encryption()
+        }
+        EncryptEncryptionMode::Transport => {
+            KeyFlags::empty().set_transport_encryption()
+        }
+        EncryptEncryptionMode::All => KeyFlags::empty()
+            .set_storage_encryption()
+            .set_transport_encryption(),
+    };
+
+    let mut signers = get_signing_keys(
+        &opts.signers, opts.policy, opts.private_key_store, opts.time, None)?;
 
     // Build a vector of recipients to hand to Encryptor.
     let mut recipient_subkeys: Vec<Recipient> = Vec::new();
-    for cert in recipients.iter() {
+    for cert in opts.recipients.iter() {
         let mut count = 0;
-        for key in cert.keys().with_policy(policy, None).alive().revoked(false)
+        for key in cert.keys().with_policy(opts.policy, None).alive().revoked(false)
             .key_flags(&mode).supported().map(|ka| ka.key())
         {
             recipient_subkeys.push(key.into());
@@ -128,7 +363,7 @@ pub fn encrypt<'a>(policy: &'a dyn Policy,
         }
         if count == 0 {
             let mut expired_keys = Vec::new();
-            for ka in cert.keys().with_policy(policy, None).revoked(false)
+            for ka in cert.keys().with_policy(opts.policy, None).revoked(false)
                 .key_flags(&mode).supported()
             {
                 let key = ka.key();
@@ -140,7 +375,7 @@ pub fn encrypt<'a>(policy: &'a dyn Policy,
             expired_keys.sort_by_key(|(expiration_time, _)| *expiration_time);
 
             if let Some((expiration_time, key)) = expired_keys.last() {
-                if use_expired_subkey {
+                if opts.use_expired_subkey {
                     recipient_subkeys.push((*key).into());
                 } else {
                     use chrono::{DateTime, offset::Utc};
@@ -160,22 +395,21 @@ pub fn encrypt<'a>(policy: &'a dyn Policy,
 
     // We want to encrypt a literal data packet.
     let encryptor =
-        Encryptor::for_recipients(message, recipient_subkeys)
+        Encryptor::for_recipients(opts.message, recipient_subkeys)
         .add_passwords(passwords);
 
     let mut sink = encryptor.build()
         .context("Failed to create encryptor")?;
 
-    match compression {
-        "none" => (),
-        "pad" => sink = Padder::new(sink).build()?,
-        "zip" => sink =
+    match opts.compression {
+        EncryptCompressionMode::None => (),
+        EncryptCompressionMode::Pad => sink = Padder::new(sink).build()?,
+        EncryptCompressionMode::Zip => sink =
             Compressor::new(sink).algo(CompressionAlgorithm::Zip).build()?,
-        "zlib" => sink =
+        EncryptCompressionMode::Zlib => sink =
             Compressor::new(sink).algo(CompressionAlgorithm::Zlib).build()?,
-        "bzip2" => sink =
+        EncryptCompressionMode::Bzip2 => sink =
             Compressor::new(sink).algo(CompressionAlgorithm::BZip2).build()?,
-        _ => unreachable!("all possible choices are handled")
     }
 
     // Optionally sign message.
@@ -183,11 +417,11 @@ pub fn encrypt<'a>(policy: &'a dyn Policy,
         let mut signer = Signer::new(sink, signers.pop().unwrap());
         for s in signers {
             signer = signer.add_signer(s);
-            if let Some(time) = time {
+            if let Some(time) = opts.time {
                 signer = signer.creation_time(time);
             }
         }
-        for r in recipients.iter() {
+        for r in opts.recipients.iter() {
             signer = signer.add_intended_recipient(r);
         }
         sink = signer.build()?;
@@ -197,7 +431,7 @@ pub fn encrypt<'a>(policy: &'a dyn Policy,
         .context("Failed to create literal writer")?;
 
     // Finally, copy stdin to our writer stack to encrypt the data.
-    io::copy(input, &mut literal_writer)
+    io::copy(opts.input, &mut literal_writer)
         .context("Failed to encrypt")?;
 
     literal_writer.finalize()
@@ -227,7 +461,7 @@ impl<'a> VHelper<'a> {
            -> Self {
         VHelper {
             config: config.clone(),
-            signatures: signatures,
+            signatures,
             certs: Some(certs),
             labels: HashMap::new(),
             trusted: HashSet::new(),
@@ -347,7 +581,7 @@ impl<'a> VerificationHelper for VHelper<'a> {
             }).collect();
 
         // Explicitly provided keys are trusted.
-        self.trusted = seen.clone();
+        self.trusted = seen;
 
         Ok(certs)
     }
@@ -414,7 +648,7 @@ pub fn split(input: &mut (dyn io::Read + Sync + Send), prefix: &str)
     let mut pos = vec![0];
 
     while let PacketParserResult::Some(pp) = ppr {
-        if let Some(ref map) = pp.map() {
+        if let Some(map) = pp.map() {
             let filename = format!(
                 "{}{}--{}{:?}", prefix,
                 pos.iter().map(|n| format!("{}", n))
@@ -449,20 +683,19 @@ pub fn split(input: &mut (dyn io::Read + Sync + Send), prefix: &str)
 }
 
 /// Joins the given files.
-pub fn join(config: Config, m: &clap::ArgMatches)
-            -> Result<()> {
+pub fn join(config: Config, c: PacketJoinCommand) -> Result<()> {
     // Either we know what kind of armor we want to produce, or we
     // need to detect it using the first packet we see.
-    let kind = parse_armor_kind(m.value_of("kind"));
-    let output = m.value_of("output");
-    let mut sink =
-        if m.is_present("binary") {
+    let kind = c.kind.into();
+    let output = c.output;
+    let mut sink = if c.binary {
+            // TODO: Does this mean kind is silently ignored if binary is given?
             // No need for any auto-detection.
-            Some(config.create_or_stdout_pgp(output,
+            Some(config.create_or_stdout_pgp(output.as_deref(),
                                              true, // Binary.
                                              armor::Kind::File)?)
         } else if let Some(kind) = kind {
-            Some(config.create_or_stdout_pgp(output,
+            Some(config.create_or_stdout_pgp(output.as_deref(),
                                              false, // Armored.
                                              kind)?)
         } else {
@@ -505,18 +738,18 @@ pub fn join(config: Config, m: &clap::ArgMatches)
         Ok(())
     }
 
-    if let Some(inputs) = m.values_of("input") {
-        for name in inputs {
+    if !c.input.is_empty() {
+        for name in c.input {
             let ppr =
                 openpgp::parse::PacketParserBuilder::from_file(name)?
                 .map(true).build()?;
-            copy(&config, ppr, output, &mut sink)?;
+            copy(&config, ppr, output.as_deref(), &mut sink)?;
         }
     } else {
         let ppr =
             openpgp::parse::PacketParserBuilder::from_reader(io::stdin())?
             .map(true).build()?;
-        copy(&config, ppr, output, &mut sink)?;
+        copy(&config, ppr, output.as_deref(), &mut sink)?;
     }
 
     sink.unwrap().finalize()?;
