@@ -6,8 +6,6 @@ use sequoia_openpgp as openpgp;
 use openpgp::{
     Result,
     KeyHandle,
-    KeyID,
-    Fingerprint,
     cert::{
         Cert,
         CertParser,
@@ -30,109 +28,93 @@ use crate::{
     serialize_keyring,
 };
 
+use crate::sq_cli::KeyserverCommand;
+use crate::sq_cli::KeyserverSubcommands;
+use crate::sq_cli::WkdCommand;
+use crate::sq_cli::WkdSubcommands;
 
-fn parse_network_policy(m: &clap::ArgMatches) -> net::Policy {
-    match m.value_of("policy").expect("has default value") {
-        "offline" => net::Policy::Offline,
-        "anonymized" => net::Policy::Anonymized,
-        "encrypted" => net::Policy::Encrypted,
-        "insecure" => net::Policy::Insecure,
-        _ => unreachable!(),
-    }
-}
-
-pub fn dispatch_keyserver(config: Config, m: &clap::ArgMatches) -> Result<()> {
-    let network_policy = parse_network_policy(m);
-    let mut ks = if let Some(uri) = m.value_of("server") {
+pub fn dispatch_keyserver(config: Config, c: KeyserverCommand) -> Result<()> {
+    let network_policy = c.network_policy.into();
+    let mut ks = if let Some(uri) = c.server {
         KeyServer::new(network_policy, &uri)
     } else {
         KeyServer::keys_openpgp_org(network_policy)
     }.context("Malformed keyserver URI")?;
 
-    let mut rt = tokio::runtime::Builder::new()
-        .basic_scheduler()
+    let rt = tokio::runtime::Builder::new_current_thread()
         .enable_io()
         .enable_time()
         .build()?;
 
-    match m.subcommand() {
-        ("get",  Some(m)) => {
-            let query = m.value_of("query").unwrap();
+    match c.subcommand {
+         KeyserverSubcommands::Get(c) => {
+            let query = c.query;
 
-            let handle: Option<KeyHandle> = {
-                let q_fp = query.parse::<Fingerprint>();
-                let q_id = query.parse::<KeyID>();
-                if let Ok(Fingerprint::V4(_)) = q_fp {
-                    q_fp.ok().map(Into::into)
-                } else if let Ok(KeyID::V4(_)) = q_id {
-                    q_fp.ok().map(Into::into)
-                } else {
-                    None
-                }
-            };
+            let handle = query.parse::<KeyHandle>();
 
-            if let Some(handle) = handle {
+            if let Ok(handle) = handle {
                 let cert = rt.block_on(ks.get(handle))
                     .context("Failed to retrieve cert")?;
 
                 let mut output =
-                    config.create_or_stdout_safe(m.value_of("output"))?;
-                if ! m.is_present("binary") {
+                    config.create_or_stdout_safe(c.output.as_deref())?;
+                if !c.binary {
                     cert.armored().serialize(&mut output)
                 } else {
                     cert.serialize(&mut output)
                 }.context("Failed to serialize cert")?;
-            } else if let Ok(Some(addr)) = UserID::from(query).email() {
+            } else if let Ok(Some(addr)) = UserID::from(query.as_str()).email() {
                 let certs = rt.block_on(ks.search(addr))
                     .context("Failed to retrieve certs")?;
 
                 let mut output =
-                    config.create_or_stdout_safe(m.value_of("output"))?;
-                serialize_keyring(&mut output, &certs,
-                                  m.is_present("binary"))?;
+                    config.create_or_stdout_safe(c.output.as_deref())?;
+                serialize_keyring(&mut output, &certs, c.binary)?;
             } else {
-                Err(anyhow::anyhow!(
+                return Err(anyhow::anyhow!(
                     "Query must be a fingerprint, a keyid, \
-                     or an email address: {:?}", query))?;
+                     or an email address: {:?}", query));
             }
         },
-        ("send",  Some(m)) => {
-            let mut input = open_or_stdin(m.value_of("input"))?;
+        KeyserverSubcommands::Send(c) => {
+            let mut input = open_or_stdin(c.input.as_deref())?;
             let cert = Cert::from_reader(&mut input).
                 context("Malformed key")?;
 
             rt.block_on(ks.send(&cert))
                 .context("Failed to send key to server")?;
         },
-        _ => unreachable!(),
     }
 
     Ok(())
 }
 
-pub fn dispatch_wkd(config: Config, m: &clap::ArgMatches) -> Result<()> {
-    let network_policy = parse_network_policy(m);
+pub fn dispatch_wkd(config: Config, c: WkdCommand) -> Result<()> {
+    let network_policy: net::Policy = c.network_policy.into();
 
-    let mut rt = tokio::runtime::Builder::new()
-        .basic_scheduler()
+    let rt = tokio::runtime::Builder::new_current_thread()
         .enable_io()
         .enable_time()
         .build()?;
 
-    match m.subcommand() {
-        ("url",  Some(m)) => {
-            let email_address = m.value_of("input").unwrap();
+    match c.subcommand {
+        WkdSubcommands::Url(c) => {
+            let email_address = c.email_address;
             let wkd_url = wkd::Url::from(email_address)?;
-            // XXX: Add other subcomand to specify whether it should be
-            // created with the advanced or the direct method.
             let url = wkd_url.to_url(None)?;
             println!("{}", url);
         },
-        ("get",  Some(m)) => {
+        WkdSubcommands::DirectUrl(c) => {
+            let email_address = c.email_address;
+            let wkd_url = wkd::Url::from(email_address)?;
+            let url = wkd_url.to_url(wkd::Variant::Direct)?;
+            println!("{}", url);
+        },
+        WkdSubcommands::Get(c) => {
             // Check that the policy allows https.
             network_policy.assert(net::Policy::Encrypted)?;
 
-            let email_address = m.value_of("input").unwrap();
+            let email_address = c.email_address;
             // XXX: EmailAddress could be created here to
             // check it's a valid email address, print the error to
             // stderr and exit.
@@ -147,30 +129,41 @@ pub fn dispatch_wkd(config: Config, m: &clap::ArgMatches) -> Result<()> {
             // But to keep the parallelism with `store export` and `keyserver get`,
             // The output is armored if not `--binary` option is given.
             let mut output =
-                config.create_or_stdout_safe(m.value_of("output"))?;
-            serialize_keyring(&mut output, &certs,
-                              m.is_present("binary"))?;
+                config.create_or_stdout_safe(c.output.as_deref())?;
+            serialize_keyring(&mut output, &certs, c.binary)?;
         },
-        ("generate", Some(m)) => {
-            let domain = m.value_of("domain").unwrap();
-            let f = open_or_stdin(m.value_of("input"))?;
-            let base_path =
-                m.value_of("base_directory").expect("required");
-            let variant = if m.is_present("direct_method") {
+        WkdSubcommands::Generate(c) => {
+            let domain = c.domain;
+            let skip = c.skip;
+            let f = open_or_stdin(c.input.as_deref())?;
+            let base_path = c.base_directory;
+            let variant = if c.direct_method {
                 wkd::Variant::Direct
             } else {
                 wkd::Variant::Advanced
             };
             let parser = CertParser::from_reader(f)?;
+            let policy = &config.policy;
             let certs: Vec<Cert> = parser.filter_map(|cert| cert.ok())
                 .collect();
             for cert in certs {
-                wkd::insert(&base_path, domain, variant, &cert)
-                    .context(format!("Failed to generate the WKD in \
-                                      {}.", base_path))?;
+                let vc = match cert.with_policy(policy, None) {
+                    Ok(vc) => vc,
+                    e @ Err(_) if !skip => e?,
+                    _ => continue,
+                };
+                if wkd::cert_contains_domain_userid(&domain, &vc) {
+                    wkd::insert(&base_path, &domain, variant, &vc)
+                        .context(format!("Failed to generate the WKD in \
+                        {}.", base_path))?;
+                } else if !skip {
+                    return Err(openpgp::Error::InvalidArgument(
+                        format!("Certificate {} does not contain User IDs in domain {}.",
+                        vc.fingerprint(), domain)
+                    ).into());
+                }
             }
         },
-        _ => unreachable!(),
     }
 
     Ok(())
