@@ -4,6 +4,7 @@ use anyhow::Context as _;
 use std::fs::OpenOptions;
 use std::io;
 use std::path::{Path, PathBuf};
+use std::str::FromStr;
 use std::time::Duration;
 use chrono::{DateTime, offset::Utc};
 use itertools::Itertools;
@@ -16,8 +17,6 @@ use openpgp::{
 };
 use crate::openpgp::{armor, Cert};
 use crate::openpgp::crypto::Password;
-use crate::openpgp::fmt::hex;
-use crate::openpgp::types::KeyFlags;
 use crate::openpgp::packet::prelude::*;
 use crate::openpgp::parse::{Parse, PacketParser, PacketParserResult};
 use crate::openpgp::packet::signature::subpacket::NotationData;
@@ -26,8 +25,14 @@ use crate::openpgp::serialize::{Serialize, stream::{Message, Armorer}};
 use crate::openpgp::cert::prelude::*;
 use crate::openpgp::policy::StandardPolicy as P;
 
+use clap::FromArgMatches;
+use crate::sq_cli::PacketSubcommands;
+use sq_cli::SqSubcommands;
+
 mod sq_cli;
 mod commands;
+mod output;
+use output::{OutputFormat, OutputVersion};
 
 fn open_or_stdin(f: Option<&str>)
                  -> Result<Box<dyn BufferedReader<()>>> {
@@ -86,7 +91,7 @@ fn parse_duration(expiry: &str) -> Result<Duration> {
         .for_each(|_| ());
     let junk = expiry.collect::<String>();
 
-    if digits == "" {
+    if digits.is_empty() {
         return Err(anyhow::anyhow!(
             "--expiry: missing count \
              (try: '2y' for 2 years)"));
@@ -106,19 +111,20 @@ fn parse_duration(expiry: &str) -> Result<Duration> {
         Some('m') | Some('M') => SECONDS_IN_YEAR / 12,
         Some('w') | Some('W') => 7 * SECONDS_IN_DAY,
         Some('d') | Some('D') => SECONDS_IN_DAY,
+        Some('s') | Some('S') => 1,
         None =>
             return Err(anyhow::anyhow!(
                 "--expiry: missing suffix \
-                 (try: '{}y', '{}m', '{}w' or '{}d' instead)",
-                digits, digits, digits, digits)),
+                 (try: '{}y', '{}m', '{}w', '{}d' or '{}s' instead)",
+                digits, digits, digits, digits, digits)),
         Some(suffix) =>
             return Err(anyhow::anyhow!(
                 "--expiry: invalid suffix '{}' \
-                 (try: '{}y', '{}m', '{}w' or '{}d' instead)",
-                suffix, digits, digits, digits, digits)),
+                 (try: '{}y', '{}m', '{}w', '{}d' or '{}s' instead)",
+                suffix, digits, digits, digits, digits, digits)),
     };
 
-    if junk != "" {
+    if !junk.is_empty() {
         return Err(anyhow::anyhow!(
             "--expiry: contains trailing junk ('{:?}') \
              (try: '{}{}')",
@@ -137,8 +143,8 @@ fn load_keys<'a, I>(files: I) -> openpgp::Result<Vec<Cert>>
         let cert = Cert::from_file(f)
             .context(format!("Failed to load key from file {:?}", f))?;
         if ! cert.is_tsk() {
-            Err(anyhow::anyhow!(
-                "Cert in file {:?} does not contain secret keys", f))?;
+            return Err(anyhow::anyhow!(
+                "Cert in file {:?} does not contain secret keys", f));
         }
         certs.push(cert);
     }
@@ -199,18 +205,6 @@ fn serialize_keyring(mut output: &mut dyn io::Write, certs: &[Cert], binary: boo
     Ok(())
 }
 
-fn parse_armor_kind(kind: Option<&str>) -> Option<armor::Kind> {
-    match kind.expect("has default value") {
-        "auto" =>    None,
-        "message" => Some(armor::Kind::Message),
-        "cert" =>    Some(armor::Kind::PublicKey),
-        "key" =>     Some(armor::Kind::SecretKey),
-        "sig" =>     Some(armor::Kind::Signature),
-        "file" =>    Some(armor::Kind::File),
-        _ => unreachable!(),
-    }
-}
-
 /// How much data to look at when detecting armor kinds.
 const ARMOR_DETECTION_LIMIT: u64 = 1 << 24;
 
@@ -218,26 +212,27 @@ const ARMOR_DETECTION_LIMIT: u64 = 1 << 24;
 ///
 /// Returns the given reader unchanged.  If the detection fails,
 /// armor::Kind::File is returned as safe default.
+#[allow(clippy::never_loop)]
 fn detect_armor_kind(input: Box<dyn BufferedReader<()>>)
                      -> (Box<dyn BufferedReader<()>>, armor::Kind) {
     let mut dup = Limitor::new(Dup::new(input), ARMOR_DETECTION_LIMIT).as_boxed();
     let kind = 'detection: loop {
-        if let Ok(ppr) = PacketParser::from_reader(&mut dup) {
-            if let PacketParserResult::Some(pp) = ppr {
-                let (packet, _) = match pp.next() {
-                    Ok(v) => v,
-                    Err(_) => break 'detection armor::Kind::File,
-                };
+        if let Ok(PacketParserResult::Some(pp)) =
+            PacketParser::from_reader(&mut dup)
+        {
+            let (packet, _) = match pp.next() {
+                Ok(v) => v,
+                Err(_) => break 'detection armor::Kind::File,
+            };
 
-                break 'detection match packet {
-                    Packet::Signature(_) => armor::Kind::Signature,
-                    Packet::SecretKey(_) => armor::Kind::SecretKey,
-                    Packet::PublicKey(_) => armor::Kind::PublicKey,
-                    Packet::PKESK(_) | Packet::SKESK(_) =>
-                        armor::Kind::Message,
-                    _ => armor::Kind::File,
-                };
-            }
+            break 'detection match packet {
+                Packet::Signature(_) => armor::Kind::Signature,
+                Packet::SecretKey(_) => armor::Kind::SecretKey,
+                Packet::PublicKey(_) => armor::Kind::PublicKey,
+                Packet::PKESK(_) | Packet::SKESK(_) =>
+                    armor::Kind::Message,
+                _ => armor::Kind::File,
+            };
         }
         break 'detection armor::Kind::File;
     };
@@ -278,7 +273,7 @@ fn decrypt_key<R>(key: Key<key::SecretParts, R>, passwords: &mut Vec<String>)
                 {
                     Ok(p) => {
                         first = false;
-                        if p == "" {
+                        if p.is_empty() {
                             // Give up.
                             break;
                         }
@@ -342,6 +337,8 @@ fn emit_unstable_cli_warning() {
 #[derive(Clone)]
 pub struct Config<'a> {
     force: bool,
+    output_format: OutputFormat,
+    output_version: Option<OutputVersion>,
     policy: P<'a>,
     /// Have we emitted the warning yet?
     unstable_cli_warning_emitted: bool,
@@ -388,38 +385,49 @@ impl Config<'_> {
     }
 }
 
+// TODO: Use `derive`d command structs. No more values_of
+// TODO: Handling (and cli position) of global arguments
 fn main() -> Result<()> {
     let policy = &mut P::new();
 
-    let matches = sq_cli::build().get_matches();
+    let c = sq_cli::SqCommand::from_arg_matches(&sq_cli::build().get_matches())?;
 
-    let known_notations: Vec<&str> = matches.values_of("known-notation")
-        .unwrap_or_default()
-        .collect();
+    let known_notations = c.known_notation
+        .iter()
+        .map(|n| n.as_str())
+        .collect::<Vec<&str>>();
     policy.good_critical_notations(&known_notations);
 
-    let force = matches.is_present("force");
+    let force = c.force;
+    let output_format = OutputFormat::from_str(&c.output_format)?;
+    let output_version = if let Some(v) = c.output_version {
+        Some(OutputVersion::from_str(&v)?)
+    } else {
+        None
+    };
 
     let mut config = Config {
         force,
+        output_format,
+        output_version,
         policy: policy.clone(),
         unstable_cli_warning_emitted: false,
     };
 
-    match matches.subcommand() {
-        ("decrypt",  Some(m)) => {
-            let mut input = open_or_stdin(m.value_of("input"))?;
+    match c.subcommand {
+        SqSubcommands::Decrypt(command) => {
+
+            let mut input = open_or_stdin(command.io.input.as_deref())?;
             let mut output =
-                config.create_or_stdout_safe(m.value_of("output"))?;
-            let certs = m.values_of("sender-cert-file")
-                .map(load_certs)
-                .unwrap_or(Ok(vec![]))?;
+                config.create_or_stdout_safe(command.io.output.as_deref())?;
+
+            let certs = load_certs(
+                command.sender_cert_file.iter().map(|s| s.as_ref()),
+            )?;
             // Fancy default for --signatures.  If you change this,
             // also change the description in the CLI definition.
-            let signatures: usize =
-                if let Some(n) = m.value_of("signatures") {
-                    n.parse()?
-                } else if certs.is_empty() {
+            let signatures = command.signatures.unwrap_or_else(|| {
+                if certs.is_empty() {
                     // No certs are given for verification, use 0 as
                     // threshold so we handle only-encrypted messages
                     // gracefully.
@@ -428,137 +436,119 @@ fn main() -> Result<()> {
                     // At least one cert given, expect at least one
                     // valid signature.
                     1
-                };
-            let secrets = m.values_of("secret-key-file")
-                .map(load_keys)
-                .unwrap_or(Ok(vec![]))?;
-            commands::decrypt(config,
+                }
+            });
+            // TODO: should this be load_keys?
+            let secrets =
+                load_certs(command.secret_key_file.iter().map(|s| s.as_ref()))?;
+            let private_key_store = command.private_key_store;
+            let session_keys = command.session_key;
+            commands::decrypt(config, private_key_store.as_deref(),
                               &mut input, &mut output,
                               signatures, certs, secrets,
-                              m.is_present("dump-session-key"),
-                              m.is_present("dump"), m.is_present("hex"))?;
+                              command.dump_session_key,
+                              session_keys,
+                              command.dump, command.hex)?;
         },
-        ("encrypt",  Some(m)) => {
-            let recipients = m.values_of("recipients-cert-file")
-                .map(load_certs)
-                .unwrap_or(Ok(vec![]))?;
-            let mut input = open_or_stdin(m.value_of("input"))?;
-            let output =
-                config.create_or_stdout_pgp(m.value_of("output"),
-                                            m.is_present("binary"),
-                                            armor::Kind::Message)?;
-            let additional_secrets = m.values_of("signer-key-file")
-                .map(load_keys)
-                .unwrap_or(Ok(vec![]))?;
-            let mode = match m.value_of("mode").expect("has default") {
-                "rest" => KeyFlags::empty()
-                    .set_storage_encryption(),
-                "transport" => KeyFlags::empty()
-                    .set_transport_encryption(),
-                "all" => KeyFlags::empty()
-                    .set_storage_encryption()
-                    .set_transport_encryption(),
-                _ => unreachable!("uses possible_values"),
-            };
-            let time = if let Some(time) = m.value_of("time") {
-                Some(parse_iso8601(time, chrono::NaiveTime::from_hms(0, 0, 0))
-                         .context(format!("Bad value passed to --time: {:?}",
-                                          time))?.into())
-            } else {
-                None
-            };
-            commands::encrypt(policy, &mut input, output,
-                              m.occurrences_of("symmetric") as usize,
-                              &recipients, additional_secrets,
-                              mode,
-                              m.value_of("compression").expect("has default"),
-                              time.into(),
-                              m.is_present("use-expired-subkey"),
+        SqSubcommands::Encrypt(command) => {
+            let recipients = load_certs(
+                command.recipients_cert_file.iter().map(|s| s.as_ref()),
             )?;
+            let mut input = open_or_stdin(command.io.input.as_deref())?;
+
+            let output = config.create_or_stdout_pgp(
+                command.io.output.as_deref(),
+                command.binary,
+                armor::Kind::Message,
+            )?;
+
+            let additional_secrets =
+                load_certs(command.signer_key_file.iter().map(|s| s.as_ref()))?;
+
+            let time = command.time.map(|t| t.time.into());
+            let private_key_store = command.private_key_store.as_deref();
+            commands::encrypt(commands::EncryptOpts {
+                policy,
+                private_key_store,
+                input: &mut input,
+                message: output,
+                npasswords: command.symmetric,
+                recipients: &recipients,
+                signers: additional_secrets,
+                mode: command.mode,
+                compression: command.compression,
+                time,
+                use_expired_subkey: command.use_expired_subkey,
+            })?;
         },
-        ("sign",  Some(m)) => {
-            let mut input = open_or_stdin(m.value_of("input"))?;
-            let output = m.value_of("output");
-            let detached = m.is_present("detached");
-            let binary = m.is_present("binary");
-            let append = m.is_present("append");
-            let notarize = m.is_present("notarize");
-            let secrets = m.values_of("secret-key-file")
-                .map(load_keys)
-                .unwrap_or(Ok(vec![]))?;
-            let time = if let Some(time) = m.value_of("time") {
-                Some(parse_iso8601(time, chrono::NaiveTime::from_hms(0, 0, 0))
-                         .context(format!("Bad value passed to --time: {:?}",
-                                          time))?.into())
-            } else {
-                None
-            };
-            // Each --notation takes two values.  The iterator
-            // returns them one at a time, however.
-            let mut notations: Vec<(bool, NotationData)> = Vec::new();
-            if let Some(mut n) = m.values_of("notation") {
-                while let Some(name) = n.next() {
-                    let value = n.next().unwrap();
+        SqSubcommands::Sign(command) => {
+            let mut input = open_or_stdin(command.io.input.as_deref())?;
+            let output = command.io.output.as_deref();
+            let detached = command.detached;
+            let binary = command.binary;
+            let append = command.append;
+            let notarize = command.notarize;
+            let private_key_store = command.private_key_store.as_deref();
+            let secrets =
+                load_certs(command.secret_key_file.iter().map(|s| s.as_ref()))?;
+            let time = command.time.map(|t| t.time.into());
 
-                    let (critical, name) = if name.len() > 0
-                        && Some('!') == name.chars().next()
-                    {
-                        (true, &name[1..])
-                    } else {
-                        (false, name)
-                    };
+            let notations = parse_notations(command.notation.unwrap_or_default())?;
 
-                    notations.push(
-                        (critical,
-                         NotationData::new(
-                             name, value,
-                             NotationDataFlags::empty().set_human_readable())));
-                }
-            }
-
-            if let Some(merge) = m.value_of("merge") {
+            if let Some(merge) = command.merge {
                 let output = config.create_or_stdout_pgp(output, binary,
                                                          armor::Kind::Message)?;
-                let mut input2 = open_or_stdin(Some(merge))?;
+                let mut input2 = open_or_stdin(Some(&merge))?;
                 commands::merge_signatures(&mut input, &mut input2, output)?;
-            } else if m.is_present("clearsign") {
+            } else if command.clearsign {
                 let output = config.create_or_stdout_safe(output)?;
-                commands::sign::clearsign(config, input, output, secrets,
+                commands::sign::clearsign(config, private_key_store, input, output, secrets,
                                           time, &notations)?;
             } else {
-                commands::sign(config, &mut input, output, secrets, detached,
-                               binary, append, notarize, time, &notations)?;
+                commands::sign(commands::sign::SignOpts {
+                    config,
+                    private_key_store,
+                    input: &mut input,
+                    output_path: output,
+                    secrets,
+                    detached,
+                    binary,
+                    append,
+                    notarize,
+                    time,
+                    notations: &notations
+                })?;
             }
         },
-        ("verify",  Some(m)) => {
-            let mut input = open_or_stdin(m.value_of("input"))?;
+        SqSubcommands::Verify(command) => {
+            // TODO: Fix interface of open_or_stdin, create_or_stdout_safe, etc.
+            let mut input = open_or_stdin(command.io.input.as_deref())?;
             let mut output =
-                config.create_or_stdout_safe(m.value_of("output"))?;
-            let mut detached = if let Some(f) = m.value_of("detached") {
+                config.create_or_stdout_safe(command.io.output.as_deref())?;
+            let mut detached = if let Some(f) = command.detached {
                 Some(File::open(f)?)
             } else {
                 None
             };
-            let signatures: usize =
-                m.value_of("signatures").expect("has a default").parse()?;
-            let certs = m.values_of("sender-cert-file")
-                .map(load_certs)
-                .unwrap_or(Ok(vec![]))?;
+            let signatures = command.signatures;
+            // TODO ugly adaptation to load_certs' signature, fix later
+            let certs = load_certs(command.sender_cert_file.iter().map(|s| s.as_ref()))?;
             commands::verify(config, &mut input,
                              detached.as_mut().map(|r| r as &mut (dyn io::Read + Sync + Send)),
                              &mut output, signatures, certs)?;
         },
 
-        ("armor",  Some(m)) => {
-            let input = open_or_stdin(m.value_of("input"))?;
-            let mut want_kind = parse_armor_kind(m.value_of("kind"));
+        // TODO: Extract body to commands/armor.rs
+        SqSubcommands::Armor(command) => {
+            let input = open_or_stdin(command.io.input.as_deref())?;
+            let mut want_kind: Option<armor::Kind> = command.kind.into();
 
             // Peek at the data.  If it looks like it is armored
             // data, avoid armoring it again.
             let mut dup = Limitor::new(Dup::new(input), ARMOR_DETECTION_LIMIT);
             let (already_armored, have_kind) = {
                 let mut reader =
-                    armor::Reader::new(&mut dup,
+                    armor::Reader::from_reader(&mut dup,
                                        armor::ReaderMode::Tolerant(None));
                 (reader.data(8).is_ok(), reader.kind())
             };
@@ -570,7 +560,7 @@ fn main() -> Result<()> {
             {
                 // It is already armored and has the correct kind.
                 let mut output =
-                    config.create_or_stdout_safe(m.value_of("output"))?;
+                    config.create_or_stdout_safe(command.io.output.as_deref())?;
                 io::copy(&mut input, &mut output)?;
                 return Ok(());
             }
@@ -585,13 +575,13 @@ fn main() -> Result<()> {
             let want_kind = want_kind.expect("given or detected");
 
             let mut output =
-                config.create_or_stdout_pgp(m.value_of("output"),
+                config.create_or_stdout_pgp(command.io.output.as_deref(),
                                             false, want_kind)?;
 
             if already_armored {
                 // Dearmor and copy to change the type.
                 let mut reader =
-                    armor::Reader::new(input,
+                    armor::Reader::from_reader(input,
                                        armor::ReaderMode::Tolerant(None));
                 io::copy(&mut reader, &mut output)?;
             } else {
@@ -599,98 +589,138 @@ fn main() -> Result<()> {
             }
             output.finalize()?;
         },
-        ("dearmor",  Some(m)) => {
-            let mut input = open_or_stdin(m.value_of("input"))?;
+        SqSubcommands::Dearmor(command) => {
+            let mut input = open_or_stdin(command.io.input.as_deref())?;
             let mut output =
-                config.create_or_stdout_safe(m.value_of("output"))?;
-            let mut filter = armor::Reader::new(&mut input, None);
+                config.create_or_stdout_safe(command.io.output.as_deref())?;
+            let mut filter = armor::Reader::from_reader(&mut input, None);
             io::copy(&mut filter, &mut output)?;
         },
         #[cfg(feature = "autocrypt")]
-        ("autocrypt", Some(m)) => commands::autocrypt::dispatch(config, m)?,
-
-        ("inspect",  Some(m)) => {
-            let mut output =
-                config.create_or_stdout_unsafe(m.value_of("output"))?;
-            commands::inspect(m, policy, &mut output)?;
+        SqSubcommands::Autocrypt(command) => {
+            commands::autocrypt::dispatch(config, &command)?;
+        },
+        SqSubcommands::Inspect(command) => {
+            // sq inspect does not have --output, but commands::inspect does.
+            // Work around this mismatch by always creating a stdout output.
+            let mut output = config.create_or_stdout_unsafe(None)?;
+            commands::inspect(command, policy, &mut output)?;
         },
 
-        ("keyring", Some(m)) => commands::keyring::dispatch(config, m)?,
+        SqSubcommands::Keyring(command) => {
+            commands::keyring::dispatch(config, command)?
+        },
 
-        ("packet", Some(m)) => match m.subcommand() {
-            ("dump",  Some(m)) => {
-                let mut input = open_or_stdin(m.value_of("input"))?;
-                let mut output =
-                    config.create_or_stdout_unsafe(m.value_of("output"))?;
-                let session_key: Option<openpgp::crypto::SessionKey> =
-                    if let Some(sk) = m.value_of("session-key") {
-                        Some(hex::decode_pretty(sk)?.into())
-                    } else {
-                        None
-                    };
+        SqSubcommands::Packet(command) => match command.subcommand {
+            PacketSubcommands::Dump(command) => {
+                let mut input = open_or_stdin(command.io.input.as_deref())?;
+                let mut output = config.create_or_stdout_unsafe(
+                    command.io.output.as_deref(),
+                )?;
+
+                let session_key = command.session_key;
                 let width = term_size::dimensions_stdout().map(|(w, _)| w);
                 commands::dump(&mut input, &mut output,
-                               m.is_present("mpis"), m.is_present("hex"),
+                               command.mpis, command.hex,
                                session_key.as_ref(), width)?;
             },
 
-            ("decrypt",  Some(m)) => {
-                let mut input = open_or_stdin(m.value_of("input"))?;
-                let mut output =
-                    config.create_or_stdout_pgp(m.value_of("output"),
-                                                m.is_present("binary"),
-                                                armor::Kind::Message)?;
-                let secrets = m.values_of("secret-key-file")
-                    .map(load_keys)
-                    .unwrap_or(Ok(vec![]))?;
+            PacketSubcommands::Decrypt(command) => {
+                let mut input = open_or_stdin(command.io.input.as_deref())?;
+                let mut output = config.create_or_stdout_pgp(
+                    command.io.output.as_deref(),
+                    command.binary,
+                    armor::Kind::Message,
+                )?;
+
+                let secrets =
+                    load_keys(command.secret_key_file.iter().map(|s| s.as_ref()))?;
+                let session_keys = command.session_key;
                 commands::decrypt::decrypt_unwrap(
                     config,
                     &mut input, &mut output,
-                    secrets, m.is_present("dump-session-key"))?;
+                    secrets,
+                    session_keys,
+                    command.dump_session_key)?;
                 output.finalize()?;
             },
 
-            ("split",  Some(m)) => {
-                let mut input = open_or_stdin(m.value_of("input"))?;
+            PacketSubcommands::Split(command) => {
+                let mut input = open_or_stdin(command.input.as_deref())?;
                 let prefix =
                 // The prefix is either specified explicitly...
-                    m.value_of("prefix").map(|p| p.to_owned())
-                    .unwrap_or(
+                    command.prefix.unwrap_or(
                         // ... or we derive it from the input file...
-                        m.value_of("input").and_then(|i| {
+                        command.input.and_then(|i| {
                             let p = PathBuf::from(i);
                             // (but only use the filename)
                             p.file_name().map(|f| String::from(f.to_string_lossy()))
                         })
                         // ... or we use a generic prefix...
-                            .unwrap_or(String::from("output"))
+                            .unwrap_or_else(|| String::from("output"))
                         // ... finally, add a hyphen to the derived prefix.
                             + "-");
                 commands::split(&mut input, &prefix)?;
             },
-            ("join",  Some(m)) => commands::join(config, m)?,
-            _ => unreachable!(),
+            PacketSubcommands::Join(command) => commands::join(config, command)?,
         },
 
-        #[cfg(feature = "net")]
-        ("keyserver",  Some(m)) =>
-            commands::net::dispatch_keyserver(config, m)?,
+        SqSubcommands::Keyserver(command) => {
+            commands::net::dispatch_keyserver(config, command)?
+        }
 
-        ("key", Some(m)) => commands::key::dispatch(config, m)?,
+        SqSubcommands::Key(command) => {
+            commands::key::dispatch(config, command)?
+        }
 
-        #[cfg(feature = "net")]
-        ("wkd",  Some(m)) => commands::net::dispatch_wkd(config, m)?,
+        SqSubcommands::Revoke(command) => {
+            commands::revoke::dispatch(config, command)?
+        }
 
-        ("certify",  Some(m)) => {
-            commands::certify::certify(config, m)?;
-        },
+        SqSubcommands::Wkd(command) => {
+            commands::net::dispatch_wkd(config, command)?
+        }
 
-        _ => unreachable!(),
+        SqSubcommands::Certify(command) => {
+            commands::certify::certify(config, command)?
+        }
     }
 
-    return Ok(())
+    Ok(())
 }
 
+fn parse_notations(n: Vec<String>) -> Result<Vec<(bool, NotationData)>> {
+
+    // TODO I'm not sure what to do about this requirement.  Setting
+    // number_of_values = 2 for the argument already makes clap bail if the
+    // length of the vec is odd.
+    assert_eq!(n.len() % 2, 0);
+
+    // Each --notation takes two values.  Iterate over them in chunks of 2.
+    let notations: Vec<(bool, NotationData)> = n
+        .chunks(2)
+        .map(|arg_pair| {
+            let name = &arg_pair[0];
+            let value = &arg_pair[1];
+
+            let (critical, name) = match name.strip_prefix('!') {
+                Some(name) => (true, name),
+                None => (false, name.as_str()),
+            };
+
+            let notation_data = NotationData::new(
+                name,
+                value,
+                NotationDataFlags::empty().set_human_readable(),
+            );
+            (critical, notation_data)
+        })
+        .collect();
+
+    Ok(notations)
+}
+
+// TODO: Replace all uses with CliTime argument type
 /// Parses the given string depicting a ISO 8601 timestamp.
 fn parse_iso8601(s: &str, pad_date_with: chrono::NaiveTime)
                  -> Result<DateTime<Utc>>
@@ -715,10 +745,8 @@ fn parse_iso8601(s: &str, pad_date_with: chrono::NaiveTime)
             if let Ok(d) = DateTime::parse_from_str(s, *f) {
                 return Ok(d.into());
             }
-        } else {
-            if let Ok(d) = chrono::NaiveDateTime::parse_from_str(s, *f) {
-                return Ok(DateTime::from_utc(d, Utc));
-            }
+        } else if let Ok(d) = chrono::NaiveDateTime::parse_from_str(s, *f) {
+            return Ok(DateTime::from_utc(d, Utc));
         }
     }
     for f in &[
