@@ -1,23 +1,29 @@
 use std::fs::{self, File};
 use std::io;
 
-use assert_cli;
-use assert_cli::Assert;
-use tempfile;
 use tempfile::TempDir;
+use assert_cmd::Command;
 
 use sequoia_openpgp as openpgp;
-use crate::openpgp::{Packet, PacketPile, Cert};
-use crate::openpgp::crypto::KeyPair;
-use crate::openpgp::packet::key::SecretKeyMaterial;
-use crate::openpgp::packet::signature::subpacket::NotationData;
-use crate::openpgp::packet::signature::subpacket::NotationDataFlags;
-use crate::openpgp::types::{CompressionAlgorithm, SignatureType};
-use crate::openpgp::parse::Parse;
-use crate::openpgp::serialize::stream::{Message, Signer, Compressor, LiteralWriter};
+use openpgp::Fingerprint;
+use openpgp::KeyHandle;
+use openpgp::Result;
+use openpgp::{Packet, PacketPile, Cert};
+use openpgp::cert::CertBuilder;
+use openpgp::crypto::KeyPair;
+use openpgp::packet::key::SecretKeyMaterial;
+use openpgp::packet::signature::subpacket::NotationData;
+use openpgp::packet::signature::subpacket::NotationDataFlags;
+use openpgp::types::{CompressionAlgorithm, SignatureType};
+use openpgp::parse::Parse;
+use openpgp::policy::StandardPolicy;
+use openpgp::serialize::stream::{Message, Signer, Compressor, LiteralWriter};
+use openpgp::serialize::Serialize;
+
+const P: &StandardPolicy = &StandardPolicy::new();
 
 fn artifact(filename: &str) -> String {
-    format!("../openpgp/tests/data/{}", filename)
+    format!("tests/data/{}", filename)
 }
 
 #[test]
@@ -26,15 +32,14 @@ fn sq_sign() {
     let sig = tmp_dir.path().join("sig0");
 
     // Sign message.
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["sign",
-              "--signer-key",
-              &artifact("keys/dennis-simon-anton-private.pgp"),
-              "--output",
-              &sig.to_string_lossy(),
-              &artifact("messages/a-cypherpunks-manifesto.txt")])
-        .unwrap();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("sign")
+        .args(["--signer-key", &artifact("keys/dennis-simon-anton-private.pgp")])
+        .args(["--output", &sig.to_string_lossy()])
+        .arg(&artifact("messages/a-cypherpunks-manifesto.txt"))
+        .assert()
+        .success();
 
     // Check that the content is sane.
     let packets: Vec<Packet> =
@@ -61,13 +66,13 @@ fn sq_sign() {
     assert!(&content[..].starts_with(b"-----BEGIN PGP MESSAGE-----\n\n"));
 
     // Verify signed message.
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["verify",
-              "--signer-cert",
-              &artifact("keys/dennis-simon-anton.pgp"),
-              &sig.to_string_lossy()])
-        .unwrap();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("verify")
+        .args(["--signer-cert", &artifact("keys/dennis-simon-anton.pgp")])
+        .arg(&*sig.to_string_lossy())
+        .assert()
+        .success();
 }
 
 #[test]
@@ -76,18 +81,17 @@ fn sq_sign_with_notations() {
     let sig = tmp_dir.path().join("sig0");
 
     // Sign message.
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["sign",
-              "--signer-key",
-              &artifact("keys/dennis-simon-anton-private.pgp"),
-              "--output",
-              &sig.to_string_lossy(),
-              "--notation", "foo", "bar",
-              "--notation", "!foo", "xyzzy",
-              "--notation", "hello@example.org", "1234567890",
-              &artifact("messages/a-cypherpunks-manifesto.txt")])
-        .unwrap();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("sign")
+        .args(["--signer-key", &artifact("keys/dennis-simon-anton-private.pgp")])
+        .args(["--output", &sig.to_string_lossy()])
+        .args(["--notation", "foo", "bar"])
+        .args(["--notation", "!foo", "xyzzy"])
+        .args(["--notation", "hello@example.org", "1234567890"])
+        .arg(&artifact("messages/a-cypherpunks-manifesto.txt"))
+        .assert()
+        .success();
 
     // Check that the content is sane.
     let packets: Vec<Packet> =
@@ -139,14 +143,14 @@ fn sq_sign_with_notations() {
     assert!(&content[..].starts_with(b"-----BEGIN PGP MESSAGE-----\n\n"));
 
     // Verify signed message.
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["--known-notation", "foo",
-              "verify",
-              "--signer-cert",
-              &artifact("keys/dennis-simon-anton.pgp"),
-              &sig.to_string_lossy()])
-        .unwrap();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .args(["--known-notation", "foo"])
+        .arg("verify")
+        .args(["--signer-cert", &artifact("keys/dennis-simon-anton.pgp")])
+        .arg(&*sig.to_string_lossy())
+        .assert()
+        .success();
 }
 
 #[test]
@@ -155,15 +159,14 @@ fn sq_sign_append() {
     let sig0 = tmp_dir.path().join("sig0");
 
     // Sign message.
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["sign",
-              "--signer-key",
-              &artifact("keys/dennis-simon-anton-private.pgp"),
-              "--output",
-              &sig0.to_string_lossy(),
-              &artifact("messages/a-cypherpunks-manifesto.txt")])
-        .unwrap();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("sign")
+        .args(["--signer-key", &artifact("keys/dennis-simon-anton-private.pgp")])
+        .args(["--output", &sig0.to_string_lossy()])
+        .arg(&artifact("messages/a-cypherpunks-manifesto.txt"))
+        .assert()
+        .success();
 
     // Check that the content is sane.
     let packets: Vec<Packet> =
@@ -190,26 +193,26 @@ fn sq_sign_append() {
     assert!(&content[..].starts_with(b"-----BEGIN PGP MESSAGE-----\n\n"));
 
     // Verify signed message.
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["verify",
-              "--signer-cert",
-              &artifact("keys/dennis-simon-anton.pgp"),
-              &sig0.to_string_lossy()])
-        .unwrap();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("verify")
+        .args(["--signer-cert", &artifact("keys/dennis-simon-anton.pgp")])
+        .arg(&*sig0.to_string_lossy())
+        .assert()
+        .success();
 
     // Now add a second signature with --append.
     let sig1 = tmp_dir.path().join("sig1");
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["sign",
-              "--append",
-              "--signer-key",
-              &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp"),
-              "--output",
-              &sig1.to_string_lossy(),
-              &sig0.to_string_lossy()])
-        .unwrap();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("sign")
+        .arg("--append")
+        .args(["--signer-key", &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp")])
+        .arg("--output")
+        .arg(&*sig1.to_string_lossy())
+        .arg(&*sig0.to_string_lossy())
+        .assert()
+        .success();
 
     // Check that the content is sane.
     let packets: Vec<Packet> =
@@ -249,28 +252,25 @@ fn sq_sign_append() {
     assert!(&content[..].starts_with(b"-----BEGIN PGP MESSAGE-----\n\n"));
 
     // Verify both signatures of the signed message.
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["verify",
-              "--signer-cert",
-              &artifact("keys/dennis-simon-anton.pgp"),
-              &sig1.to_string_lossy()])
-        .unwrap();
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["verify",
-              "--signer-cert",
-              &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp"),
-              &sig1.to_string_lossy()])
-        .unwrap();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("verify")
+        .args(["--signer-cert", &artifact("keys/dennis-simon-anton.pgp")])
+        .arg(&*sig1.to_string_lossy())
+        .assert()
+        .success();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("verify")
+        .args(["--signer-cert", &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp")])
+        .arg(&*sig1.to_string_lossy())
+        .assert()
+        .success();
 }
 
 #[test]
 #[allow(unreachable_code)]
 fn sq_sign_append_on_compress_then_sign() {
-    use crate::openpgp::policy::StandardPolicy as P;
-
-    let p = &P::new();
     let tmp_dir = TempDir::new().unwrap();
     let sig0 = tmp_dir.path().join("sig0");
 
@@ -278,7 +278,7 @@ fn sq_sign_append_on_compress_then_sign() {
     // message by foot.
     let tsk = Cert::from_file(&artifact("keys/dennis-simon-anton-private.pgp"))
         .unwrap();
-    let key = tsk.keys().with_policy(p, None).for_signing().nth(0).unwrap().key();
+    let key = tsk.keys().with_policy(P, None).for_signing().next().unwrap().key();
     let sec = match key.optional_secret() {
         Some(SecretKeyMaterial::Unencrypted(ref u)) => u.clone(),
         _ => unreachable!(),
@@ -320,27 +320,26 @@ fn sq_sign_append_on_compress_then_sign() {
     }
 
     // Verify signed message.
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["verify",
-              "--signer-cert",
-              &artifact("keys/dennis-simon-anton.pgp"),
-              &sig0.to_string_lossy()])
-        .unwrap();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("verify")
+        .args(["--signer-cert", &artifact("keys/dennis-simon-anton.pgp")])
+        .arg(&*sig0.to_string_lossy())
+        .assert()
+        .success();
 
     // Now add a second signature with --append.
     let sig1 = tmp_dir.path().join("sig1");
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["sign",
-              "--append",
-              "--signer-key",
-              &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp"),
-              "--output",
-              &sig1.to_string_lossy(),
-              &sig0.to_string_lossy()])
-        .fails() // XXX: Currently, this is not implemented.
-        .unwrap();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("sign")
+        .arg("--append")
+        .args(["--signer-key", &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp")])
+        .arg("--output")
+        .arg(&*sig1.to_string_lossy())
+        .arg(&*sig0.to_string_lossy())
+        .assert()
+        .failure(); // XXX: Currently, this is not implemented.
 
     // XXX: Currently, this is not implemented in sq.
     return;
@@ -383,20 +382,21 @@ fn sq_sign_append_on_compress_then_sign() {
     assert!(&content[..].starts_with(b"-----BEGIN PGP MESSAGE-----\n\n"));
 
     // Verify both signatures of the signed message.
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["verify",
-              "--signer-cert",
-              &artifact("keys/dennis-simon-anton.pgp"),
-              &sig0.to_string_lossy()])
-        .unwrap();
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["verify",
-              "--signer-cert",
-              &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp"),
-              &sig0.to_string_lossy()])
-        .unwrap();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("verify")
+        .args(["--signer-cert", &artifact("keys/dennis-simon-anton.pgp")])
+        .arg(&*sig0.to_string_lossy())
+        .assert()
+        .success();
+
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("verify")
+        .args(["--signer-cert", &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp")])
+        .arg(&*sig0.to_string_lossy())
+        .assert()
+        .success();
 }
 
 #[test]
@@ -405,16 +405,15 @@ fn sq_sign_detached() {
     let sig = tmp_dir.path().join("sig0");
 
     // Sign detached.
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["sign",
-              "--detached",
-              "--signer-key",
-              &artifact("keys/dennis-simon-anton-private.pgp"),
-              "--output",
-              &sig.to_string_lossy(),
-              &artifact("messages/a-cypherpunks-manifesto.txt")])
-        .unwrap();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("sign")
+        .arg("--detached")
+        .args(["--signer-key", &artifact("keys/dennis-simon-anton-private.pgp")])
+        .args(["--output", &sig.to_string_lossy()])
+        .arg(&artifact("messages/a-cypherpunks-manifesto.txt"))
+        .assert()
+        .success();
 
     // Check that the content is sane.
     let packets: Vec<Packet> =
@@ -430,15 +429,14 @@ fn sq_sign_detached() {
     assert!(&content[..].starts_with(b"-----BEGIN PGP SIGNATURE-----\n\n"));
 
     // Verify detached.
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["verify",
-              "--signer-cert",
-              &artifact("keys/dennis-simon-anton.pgp"),
-              "--detached",
-              &sig.to_string_lossy(),
-              &artifact("messages/a-cypherpunks-manifesto.txt")])
-        .unwrap();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("verify")
+        .args(["--signer-cert", &artifact("keys/dennis-simon-anton.pgp")])
+        .args(["--detached", &sig.to_string_lossy()])
+        .arg(&artifact("messages/a-cypherpunks-manifesto.txt"))
+        .assert()
+        .success();
 }
 
 #[test]
@@ -447,16 +445,15 @@ fn sq_sign_detached_append() {
     let sig = tmp_dir.path().join("sig0");
 
     // Sign detached.
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["sign",
-              "--detached",
-              "--signer-key",
-              &artifact("keys/dennis-simon-anton-private.pgp"),
-              "--output",
-              &sig.to_string_lossy(),
-              &artifact("messages/a-cypherpunks-manifesto.txt")])
-        .unwrap();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("sign")
+        .arg("--detached")
+        .args(["--signer-key", &artifact("keys/dennis-simon-anton-private.pgp")])
+        .args(["--output", &sig.to_string_lossy()])
+        .arg(&artifact("messages/a-cypherpunks-manifesto.txt"))
+        .assert()
+        .success();
 
     // Check that the content is sane.
     let packets: Vec<Packet> =
@@ -472,41 +469,37 @@ fn sq_sign_detached_append() {
     assert!(&content[..].starts_with(b"-----BEGIN PGP SIGNATURE-----\n\n"));
 
     // Verify detached.
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["verify",
-              "--signer-cert",
-              &artifact("keys/dennis-simon-anton.pgp"),
-              "--detached",
-              &sig.to_string_lossy(),
-              &artifact("messages/a-cypherpunks-manifesto.txt")])
-        .unwrap();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("verify")
+        .args(["--signer-cert", &artifact("keys/dennis-simon-anton.pgp")])
+        .args(["--detached", &sig.to_string_lossy()])
+        .arg(&artifact("messages/a-cypherpunks-manifesto.txt"))
+        .assert()
+        .success();
 
     // Check that we don't blindly overwrite signatures.
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["sign",
-              "--detached",
-              "--signer-key",
-              &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp"),
-              "--output",
-              &sig.to_string_lossy(),
-              &artifact("messages/a-cypherpunks-manifesto.txt")])
-        .fails()
-        .unwrap();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("sign")
+        .arg("--detached")
+        .args(["--signer-key", &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp")])
+        .args(["--output", &sig.to_string_lossy()])
+        .arg(&artifact("messages/a-cypherpunks-manifesto.txt"))
+        .assert()
+        .failure();
 
     // Now add a second signature with --append.
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["sign",
-              "--detached",
-              "--append",
-              "--signer-key",
-              &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp"),
-              "--output",
-              &sig.to_string_lossy(),
-              &artifact("messages/a-cypherpunks-manifesto.txt")])
-        .unwrap();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("sign")
+        .arg("--detached")
+        .arg("--append")
+        .args(["--signer-key", &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp")])
+        .args(["--output", &sig.to_string_lossy()])
+        .arg(&artifact("messages/a-cypherpunks-manifesto.txt"))
+        .assert()
+        .success();
 
     // Check that the content is sane.
     let packets: Vec<Packet> =
@@ -527,40 +520,37 @@ fn sq_sign_detached_append() {
     assert!(&content[..].starts_with(b"-----BEGIN PGP SIGNATURE-----\n\n"));
 
     // Verify both detached signatures.
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["verify",
-              "--signer-cert",
-              &artifact("keys/dennis-simon-anton.pgp"),
-              "--detached",
-              &sig.to_string_lossy(),
-              &artifact("messages/a-cypherpunks-manifesto.txt")])
-        .unwrap();
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["verify",
-              "--signer-cert",
-              &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp"),
-              "--detached",
-              &sig.to_string_lossy(),
-              &artifact("messages/a-cypherpunks-manifesto.txt")])
-        .unwrap();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("verify")
+        .args(["--signer-cert", &artifact("keys/dennis-simon-anton.pgp")])
+        .args(["--detached", &sig.to_string_lossy()])
+        .arg(&artifact("messages/a-cypherpunks-manifesto.txt"))
+        .assert()
+        .success();
+
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("verify")
+        .args(["--signer-cert", &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp")])
+        .args(["--detached", &sig.to_string_lossy()])
+        .arg(&artifact("messages/a-cypherpunks-manifesto.txt"))
+        .assert()
+        .success();
 
     // Finally, check that we don't truncate the file if something
     // goes wrong.
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["sign",
-              "--detached",
-              "--append",
-              "--signer-key",
-              // Not a private key => signing will fail.
-              &artifact("keys/erika-corinna-daniela-simone-antonia-nistp521.pgp"),
-              "--output",
-              &sig.to_string_lossy(),
-              &artifact("messages/a-cypherpunks-manifesto.txt")])
-        .fails()
-        .unwrap();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("sign")
+        .arg("--detached")
+        .arg("--append")
+        .arg("--signer-key") // Not a private key => signing will fail.
+        .arg(&artifact("keys/erika-corinna-daniela-simone-antonia-nistp521.pgp"))
+        .args(["--output", &sig.to_string_lossy()])
+        .arg(&artifact("messages/a-cypherpunks-manifesto.txt"))
+        .assert()
+        .failure();
 
     // Check that the content is still sane.
     let packets: Vec<Packet> =
@@ -586,16 +576,15 @@ fn sq_sign_append_a_notarization() {
     let sig0 = tmp_dir.path().join("sig0");
 
     // Now add a third signature with --append to a notarized message.
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["sign",
-              "--append",
-              "--signer-key",
-              &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp"),
-              "--output",
-              &sig0.to_string_lossy(),
-              &artifact("messages/signed-1-notarized-by-ed25519.pgp")])
-        .unwrap();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("sign")
+        .arg("--append")
+        .args(["--signer-key", &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp")])
+        .args(["--output", &sig0.to_string_lossy()])
+        .arg(&artifact("messages/signed-1-notarized-by-ed25519.pgp"))
+        .assert()
+        .success();
 
     // Check that the content is sane.
     let packets: Vec<Packet> =
@@ -647,27 +636,27 @@ fn sq_sign_append_a_notarization() {
     assert!(&content[..].starts_with(b"-----BEGIN PGP MESSAGE-----\n\n"));
 
     // Verify both notarizations and the signature.
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["verify",
-              "--signer-cert",
-              &artifact("keys/neal.pgp"),
-              &sig0.to_string_lossy()])
-        .unwrap();
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["verify",
-              "--signer-cert",
-              &artifact("keys/emmelie-dorothea-dina-samantha-awina-ed25519.pgp"),
-              &sig0.to_string_lossy()])
-        .unwrap();
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["verify",
-              "--signer-cert",
-              &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp"),
-              &sig0.to_string_lossy()])
-        .unwrap();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("verify")
+        .args(["--signer-cert", &artifact("keys/neal.pgp")])
+        .arg(&*sig0.to_string_lossy())
+        .assert()
+        .success();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("verify")
+        .args(["--signer-cert", &artifact("keys/emmelie-dorothea-dina-samantha-awina-ed25519.pgp")])
+        .arg(&*sig0.to_string_lossy())
+        .assert()
+        .success();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("verify")
+        .args(["--signer-cert", &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp")])
+        .arg(&*sig0.to_string_lossy())
+        .assert()
+        .success();
 }
 
 #[test]
@@ -676,16 +665,15 @@ fn sq_sign_notarize() {
     let sig0 = tmp_dir.path().join("sig0");
 
     // Now add a third signature with --append to a notarized message.
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["sign",
-              "--notarize",
-              "--signer-key",
-              &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp"),
-              "--output",
-              &sig0.to_string_lossy(),
-              &artifact("messages/signed-1.gpg")])
-        .unwrap();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("sign")
+        .arg("--notarize")
+        .args(["--signer-key", &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp")])
+        .args(["--output", &sig0.to_string_lossy()])
+        .arg(&artifact("messages/signed-1.gpg"))
+        .assert()
+        .success();
 
     // Check that the content is sane.
     let packets: Vec<Packet> =
@@ -725,20 +713,20 @@ fn sq_sign_notarize() {
     assert!(&content[..].starts_with(b"-----BEGIN PGP MESSAGE-----\n\n"));
 
     // Verify both notarizations and the signature.
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["verify",
-              "--signer-cert",
-              &artifact("keys/neal.pgp"),
-              &sig0.to_string_lossy()])
-        .unwrap();
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["verify",
-              "--signer-cert",
-              &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp"),
-              &sig0.to_string_lossy()])
-        .unwrap();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("verify")
+        .args(["--signer-cert", &artifact("keys/neal.pgp")])
+        .arg(&*sig0.to_string_lossy())
+        .assert()
+        .success();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("verify")
+        .args(["--signer-cert", &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp")])
+        .arg(&*sig0.to_string_lossy())
+        .assert()
+        .success();
 }
 
 #[test]
@@ -747,16 +735,15 @@ fn sq_sign_notarize_a_notarization() {
     let sig0 = tmp_dir.path().join("sig0");
 
     // Now add a third signature with --append to a notarized message.
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["sign",
-              "--notarize",
-              "--signer-key",
-              &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp"),
-              "--output",
-              &sig0.to_string_lossy(),
-              &artifact("messages/signed-1-notarized-by-ed25519.pgp")])
-        .unwrap();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("sign")
+        .arg("--notarize")
+        .args(["--signer-key", &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256-private.pgp")])
+        .args(["--output", &sig0.to_string_lossy()])
+        .arg(&artifact("messages/signed-1-notarized-by-ed25519.pgp"))
+        .assert()
+        .success();
 
     // Check that the content is sane.
     let packets: Vec<Packet> =
@@ -808,25 +795,95 @@ fn sq_sign_notarize_a_notarization() {
     assert!(&content[..].starts_with(b"-----BEGIN PGP MESSAGE-----\n\n"));
 
     // Verify both notarizations and the signature.
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["verify",
-              "--signer-cert",
-              &artifact("keys/neal.pgp"),
-              &sig0.to_string_lossy()])
-        .unwrap();
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["verify",
-              "--signer-cert",
-              &artifact("keys/emmelie-dorothea-dina-samantha-awina-ed25519.pgp"),
-              &sig0.to_string_lossy()])
-        .unwrap();
-    Assert::cargo_binary("sq")
-        .with_args(
-            &["verify",
-              "--signer-cert",
-              &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp"),
-              &sig0.to_string_lossy()])
-        .unwrap();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("verify")
+        .args(["--signer-cert", &artifact("keys/neal.pgp")])
+        .arg(&*sig0.to_string_lossy())
+        .assert()
+        .success();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("verify")
+        .args(["--signer-cert", &artifact("keys/emmelie-dorothea-dina-samantha-awina-ed25519.pgp")])
+        .arg(&*sig0.to_string_lossy())
+        .assert()
+        .success();
+    Command::cargo_bin("sq")
+        .unwrap()
+        .arg("verify")
+        .args(["--signer-cert", &artifact("keys/erika-corinna-daniela-simone-antonia-nistp256.pgp")])
+        .arg(&*sig0.to_string_lossy())
+        .assert()
+        .success();
+}
+
+#[test]
+fn sq_multiple_signers() -> Result<()> {
+    let tmp = TempDir::new()?;
+
+    let gen = |userid: &str| {
+        CertBuilder::new()
+            .add_signing_subkey()
+            .add_userid(userid)
+            .generate().map(|(key, _rev)| key)
+    };
+
+    let alice = gen("<alice@some.org>")?;
+    let alice_pgp = tmp.path().join("alice.pgp");
+    let mut file = File::create(&alice_pgp)?;
+    alice.as_tsk().serialize(&mut file)?;
+
+    let bob = gen("<bob@some.org>")?;
+    let bob_pgp = tmp.path().join("bob.pgp");
+    let mut file = File::create(&bob_pgp)?;
+    bob.as_tsk().serialize(&mut file)?;
+
+    // Sign message.
+    let assertion = Command::cargo_bin("sq")?
+        .args([
+            "sign",
+            "--signer-key", alice_pgp.to_str().unwrap(),
+            "--signer-key", &bob_pgp.to_str().unwrap(),
+            "--detached",
+        ])
+        .write_stdin(&b"foo"[..])
+        .assert().try_success()?;
+
+    let stdout = String::from_utf8_lossy(&assertion.get_output().stdout);
+
+    let pp = PacketPile::from_bytes(&*stdout)?;
+
+    assert_eq!(pp.children().count(), 2,
+               "expected two packets");
+
+    let mut sigs: Vec<Fingerprint> = pp.children().map(|p| {
+        if let &Packet::Signature(ref sig) = p {
+            if let Some(KeyHandle::Fingerprint(fpr))
+                = sig.get_issuers().into_iter().next()
+            {
+                fpr
+            } else {
+                panic!("No issuer fingerprint subpacket!");
+            }
+        } else {
+            panic!("Expected a signature, got: {:?}", pp);
+        }
+    }).collect();
+    sigs.sort();
+
+    let alice_sig_fpr = alice.with_policy(P, None)?
+        .keys().for_signing().next().unwrap().fingerprint();
+    let bob_sig_fpr = bob.with_policy(P, None)?
+        .keys().for_signing().next().unwrap().fingerprint();
+
+    let mut expected = vec![
+        alice_sig_fpr,
+        bob_sig_fpr,
+    ];
+    expected.sort();
+
+    assert_eq!(sigs, expected);
+
+    Ok(())
 }
